@@ -14,7 +14,6 @@ except ImportError:
     md5new = md5.new
 if sys.version_info[:2] < (2, 6):
     from sets import Set as set
-
 import textwrap
 
 from os.path import join
@@ -44,6 +43,10 @@ API_FILES = [join('multiarray', 'methods.c'),
              join('multiarray', 'refcount.c'),
              join('multiarray', 'conversion_utils.c'),
              join('multiarray', 'buffer.c'),
+             join('multiarray', 'datetime.c'),
+             join('multiarray', 'nditer.c.src'),
+             join('multiarray', 'nditer_pywrap.c'),
+             join('multiarray', 'einsum.c.src'),
              join('umath', 'ufunc_object.c'),
              join('umath', 'loops.c.src'),
             ]
@@ -57,7 +60,7 @@ def remove_whitespace(s):
     return ''.join(s.split())
 
 def _repl(str):
-    return str.replace('intp', 'npy_intp').replace('Bool','npy_bool')
+    return str.replace('Bool','npy_bool')
 
 class Function(object):
     def __init__(self, name, return_type, args, doc=''):
@@ -66,14 +69,14 @@ class Function(object):
         self.args = args
         self.doc = doc
 
-    def _format_arg(self, (typename, name)):
+    def _format_arg(self, typename, name):
         if typename.endswith('*'):
             return typename + name
         else:
             return typename + ' ' + name
 
     def __str__(self):
-        argstr = ', '.join([self._format_arg(a) for a in self.args])
+        argstr = ', '.join([self._format_arg(*a) for a in self.args])
         if self.doc:
             doccomment = '/* %s */\n' % self.doc
         else:
@@ -82,7 +85,7 @@ class Function(object):
 
     def to_ReST(self):
         lines = ['::', '', '  ' + self.return_type]
-        argstr = ',\000'.join([self._format_arg(a) for a in self.args])
+        argstr = ',\000'.join([self._format_arg(*a) for a in self.args])
         name = '  %s' % (self.name,)
         s = textwrap.wrap('(%s)' % (argstr,), width=72,
                           initial_indent=name,
@@ -238,7 +241,7 @@ def find_functions(filename, tag='API'):
                 else:
                     function_args.append(line)
         except:
-            print filename, lineno+1
+            print(filename, lineno+1)
             raise
     fo.close()
     return functions
@@ -371,9 +374,9 @@ NPY_NO_EXPORT %s %s \\\n       (%s);""" % (self.return_type,
 def order_dict(d):
     """Order dict by its values."""
     o = d.items()
-    def cmp(x, y):
-        return x[1] - y[1]
-    return sorted(o, cmp=cmp)
+    def _key(x):
+        return (x[1], x[0])
+    return sorted(o, key=_key)
 
 def merge_api_dicts(dicts):
     ret = {}
@@ -432,12 +435,12 @@ def fullapi_hash(api_dicts):
         def sorted_by_values(d):
             """Sort a dictionary by its values. Assume the dictionary items is of
             the form func_name -> order"""
-            return sorted(d.items(), key=lambda (x, y): (y, x))
+            return sorted(d.items(), key=lambda x_y: (x_y[1], x_y[0]))
         for name, index in sorted_by_values(d):
             a.extend(name)
             a.extend(str(index))
 
-    return md5new(''.join(a)).hexdigest()
+    return md5new(''.join(a).encode('ascii')).hexdigest()
 
 # To parse strings like 'hex = checksum' where hex is e.g. 0x1234567F and
 # checksum a 128 bits md5 checksum (hex format as well)
@@ -464,11 +467,11 @@ def main():
     functions = get_api_functions(tagname, order_file)
     m = md5new(tagname)
     for func in functions:
-        print func
+        print(func)
         ah = func.api_hash()
         m.update(ah)
-        print hex(int(ah,16))
-    print hex(int(m.hexdigest()[:8],16))
+        print(hex(int(ah,16)))
+    print(hex(int(m.hexdigest()[:8],16)))
 
 if __name__ == '__main__':
     main()
