@@ -44,6 +44,11 @@ array_shape_set(PyArrayObject *self, PyObject *val)
     int nd;
     PyObject *ret;
 
+    if (val == NULL) {
+        PyErr_SetString(PyExc_AttributeError,
+                "Cannot delete array shape");
+        return -1;
+    }
     /* Assumes C-order */
     ret = PyArray_Reshape(self, val);
     if (ret == NULL) {
@@ -99,6 +104,11 @@ array_strides_set(PyArrayObject *self, PyObject *obj)
     Py_ssize_t buf_len;
     char *buf;
 
+    if (obj == NULL) {
+        PyErr_SetString(PyExc_AttributeError,
+                "Cannot delete array strides");
+        return -1;
+    }
     if (!PyArray_IntpConverter(obj, &newstrides) ||
         newstrides.ptr == NULL) {
         PyErr_SetString(PyExc_TypeError, "invalid strides");
@@ -305,6 +315,11 @@ array_data_set(PyArrayObject *self, PyObject *op)
     Py_ssize_t buf_len;
     int writeable=1;
 
+    if (op == NULL) {
+        PyErr_SetString(PyExc_AttributeError,
+                "Cannot delete array data");
+        return -1;
+    }
     if (PyObject_AsWriteBuffer(op, &buf, &buf_len) < 0) {
         writeable = 0;
         if (PyObject_AsReadBuffer(op, (const void **)&buf, &buf_len) < 0) {
@@ -401,6 +416,12 @@ array_descr_set(PyArrayObject *self, PyObject *arg)
     int index;
     char *msg = "new type not compatible with array.";
 
+    if (arg == NULL) {
+        PyErr_SetString(PyExc_AttributeError,
+                "Cannot delete array dtype");
+        return -1;
+    }
+
     if (!(PyArray_DescrConverter(arg, &newtype)) ||
         newtype == NULL) {
         PyErr_SetString(PyExc_TypeError, "invalid data-type for array");
@@ -665,6 +686,11 @@ array_real_set(PyArrayObject *self, PyObject *val)
     PyArrayObject *new;
     int rint;
 
+    if (val == NULL) {
+        PyErr_SetString(PyExc_AttributeError,
+                "Cannot delete array real part");
+        return -1;
+    }
     if (PyArray_ISCOMPLEX(self)) {
         ret = _get_part(self, 0);
         if (ret == NULL) {
@@ -722,6 +748,11 @@ array_imag_get(PyArrayObject *self)
 static int
 array_imag_set(PyArrayObject *self, PyObject *val)
 {
+    if (val == NULL) {
+        PyErr_SetString(PyExc_AttributeError,
+                "Cannot delete array imaginary part");
+        return -1;
+    }
     if (PyArray_ISCOMPLEX(self)) {
         PyArrayObject *ret;
         PyArrayObject *new;
@@ -742,8 +773,8 @@ array_imag_set(PyArrayObject *self, PyObject *val)
         return rint;
     }
     else {
-        PyErr_SetString(PyExc_TypeError, "array does not have "\
-                        "imaginary part to set");
+        PyErr_SetString(PyExc_TypeError,
+                "array does not have imaginary part to set");
         return -1;
     }
 }
@@ -764,7 +795,12 @@ array_flat_set(PyArrayObject *self, PyObject *val)
     int swap;
     PyArray_CopySwapFunc *copyswap;
 
-    typecode = self->descr;
+    if (val == NULL) {
+        PyErr_SetString(PyExc_AttributeError,
+                "Cannot delete array flat iterator");
+        return -1;
+    }
+    typecode = PyArray_DESCR(self);
     Py_INCREF(typecode);
     arr = PyArray_FromAny(val, typecode,
                           0, 0, FORCECAST | FORTRAN_IF(self), NULL);
@@ -896,7 +932,7 @@ NPY_NO_EXPORT PyGetSetDef array_getsetlist[] = {
         (getter)array_ctypes_get,
         NULL,
         NULL, NULL},
-    {   "T",
+    {"T",
         (getter)array_transpose_get,
         NULL,
         NULL, NULL},
