@@ -218,7 +218,7 @@ PyArray_GetMap(PyArrayMapIterObject *mit)
 
     PyArrayObject *ret, *temp;
     PyArrayIterObject *it;
-    int index;
+    npy_intp index;
     int swap;
     PyArray_CopySwapFunc *copyswap;
 
@@ -278,7 +278,7 @@ PyArray_SetMap(PyArrayMapIterObject *mit, PyObject *op)
 {
     PyObject *arr = NULL;
     PyArrayIterObject *it;
-    int index;
+    npy_intp index;
     int swap;
     PyArray_CopySwapFunc *copyswap;
     PyArray_Descr *descr;
@@ -813,24 +813,43 @@ array_ass_sub(PyArrayObject *self, PyObject *index, PyObject *op)
             }
         }
 
+#if defined(NPY_PY3K)
+        PyErr_Format(PyExc_ValueError,
+                     "field named %S not found.",
+                     index);
+#else
         PyErr_Format(PyExc_ValueError,
                      "field named %s not found.",
                      PyString_AsString(index));
+#endif
         return -1;
     }
 
+    if (index == Py_Ellipsis) {
+        /*
+         * Doing "a[...] += 1" triggers assigning an array to itself,
+         * so this check is needed.
+         */
+        if ((PyObject *)self == op) {
+            return 0;
+        }
+        else {
+            return PyArray_CopyObject(self, op);
+        }
+    }
+
     if (self->nd == 0) {
         /*
          * Several different exceptions to the 0-d no-indexing rule
          *
-         *  1) ellipses
+         *  1) ellipses (handled above generally)
          *  2) empty tuple
          *  3) Using newaxis (None)
          *  4) Boolean mask indexing
          */
-        if (index == Py_Ellipsis || index == Py_None ||
-            (PyTuple_Check(index) && (0 == PyTuple_GET_SIZE(index) ||
-                                      count_new_axes_0d(index) > 0))) {
+        if (index == Py_None || (PyTuple_Check(index) &&
+                                        (0 == PyTuple_GET_SIZE(index) ||
+                                         count_new_axes_0d(index) > 0))) {
             return self->descr->f->setitem(op, self->data, self);
         }
         if (PyBool_Check(index) || PyArray_IsScalar(index, Bool) ||
@@ -1690,5 +1709,3 @@ NPY_NO_EXPORT PyTypeObject PyArrayMapIter_Type = {
 };
 
 /** END of Subscript Iterator **/
-
-
