@@ -1,11 +1,14 @@
 import sys
+import platform
 from decimal import Decimal
 
 import numpy as np
 from numpy.core import *
 from numpy.random import rand, randint, randn
 from numpy.testing import *
+from numpy.testing.utils import WarningManager
 from numpy.core.multiarray import dot as dot_
+import warnings
 
 class Vec:
     def __init__(self,sequence=None):
@@ -121,7 +124,7 @@ class TestDot(TestCase):
                 arg2 = rand(*dim2)
                 c1 = dot(arg1, arg2)
                 c2 = dot_(arg1, arg2)
-                assert (c1.shape == c2.shape)
+                assert_(c1.shape == c2.shape)
                 assert_almost_equal(c1, c2, decimal=self.N)
 
     def test_vecobject(self):
@@ -151,28 +154,27 @@ class TestResize(TestCase):
         Ar = resize(A, (0,))
         assert_equal(Ar, array([]))
 
-
 class TestNonarrayArgs(TestCase):
     # check that non-array arguments to functions wrap them in arrays
     def test_squeeze(self):
         A = [[[1,1,1],[2,2,2],[3,3,3]]]
-        assert squeeze(A).shape == (3,3)
+        assert_(squeeze(A).shape == (3,3))
 
     def test_cumproduct(self):
         A = [[1,2,3],[4,5,6]]
-        assert all(cumproduct(A) == array([1,2,6,24,120,720]))
+        assert_(all(cumproduct(A) == array([1,2,6,24,120,720])))
 
     def test_size(self):
         A = [[1,2,3],[4,5,6]]
-        assert size(A) == 6
-        assert size(A,0) == 2
-        assert size(A,1) == 3
+        assert_(size(A) == 6)
+        assert_(size(A,0) == 2)
+        assert_(size(A,1) == 3)
 
     def test_mean(self):
         A = [[1,2,3],[4,5,6]]
-        assert mean(A) == 3.5
-        assert all(mean(A,0) == array([2.5,3.5,4.5]))
-        assert all(mean(A,1) == array([2.,5.]))
+        assert_(mean(A) == 3.5)
+        assert_(all(mean(A,0) == array([2.5,3.5,4.5])))
+        assert_(all(mean(A,1) == array([2.,5.])))
 
     def test_std(self):
         A = [[1,2,3],[4,5,6]]
@@ -221,16 +223,25 @@ class TestBoolScalar(TestCase):
 
 
 class TestSeterr(TestCase):
+    def test_default(self):
+        err = geterr()
+        self.assertEqual(err, dict(
+            divide='warn',
+            invalid='warn',
+            over='warn',
+            under='ignore',
+        ))
+
     def test_set(self):
         err = seterr()
         try:
-            old = seterr(divide='warn')
+            old = seterr(divide='print')
             self.assertTrue(err == old)
             new = seterr()
-            self.assertTrue(new['divide'] == 'warn')
+            self.assertTrue(new['divide'] == 'print')
             seterr(over='raise')
             self.assertTrue(geterr()['over'] == 'raise')
-            self.assertTrue(new['divide'] == 'warn')
+            self.assertTrue(new['divide'] == 'print')
             seterr(**old)
             self.assertTrue(geterr() == old)
         finally:
@@ -251,6 +262,256 @@ class TestSeterr(TestCase):
             seterr(**err)
 
 
+class TestFloatExceptions(TestCase):
+    def assert_raises_fpe(self, fpeerr, flop, x, y):
+        ftype = type(x)
+        try:
+            flop(x, y)
+            assert_(False,
+                    "Type %s did not raise fpe error '%s'." % (ftype, fpeerr))
+        except FloatingPointError, exc:
+            assert_(str(exc).find(fpeerr) >= 0,
+                    "Type %s raised wrong fpe error '%s'." % (ftype, exc))
+
+    def assert_op_raises_fpe(self, fpeerr, flop, sc1, sc2):
+        """Check that fpe exception is raised.
+
+       Given a floating operation `flop` and two scalar values, check that
+       the operation raises the floating point exception specified by
+       `fpeerr`. Tests all variants with 0-d array scalars as well.
+
+        """
+        self.assert_raises_fpe(fpeerr, flop, sc1, sc2);
+        self.assert_raises_fpe(fpeerr, flop, sc1[()], sc2);
+        self.assert_raises_fpe(fpeerr, flop, sc1, sc2[()]);
+        self.assert_raises_fpe(fpeerr, flop, sc1[()], sc2[()]);
+
+    @dec.knownfailureif(True, "See ticket 1755")
+    def test_floating_exceptions(self):
+        """Test basic arithmetic function errors"""
+        oldsettings = np.seterr(all='raise')
+        try:
+            # Test for all real and complex float types
+            for typecode in np.typecodes['AllFloat']:
+                ftype = np.obj2sctype(typecode)
+                if np.dtype(ftype).kind == 'f':
+                    # Get some extreme values for the type
+                    fi = np.finfo(ftype)
+                    ft_tiny = fi.tiny
+                    ft_max = fi.max
+                    ft_eps = fi.eps
+                    underflow = 'underflow'
+                    divbyzero = 'divide by zero'
+                else:
+                    # 'c', complex, corresponding real dtype
+                    rtype = type(ftype(0).real)
+                    fi = np.finfo(rtype)
+                    ft_tiny = ftype(fi.tiny)
+                    ft_max = ftype(fi.max)
+                    ft_eps = ftype(fi.eps)
+                    # The complex types raise different exceptions
+                    underflow = ''
+                    divbyzero = ''
+                overflow = 'overflow'
+                invalid = 'invalid'
+
+                self.assert_raises_fpe(underflow,
+                        lambda a,b:a/b, ft_tiny, ft_max)
+                self.assert_raises_fpe(underflow,
+                        lambda a,b:a*b, ft_tiny, ft_tiny)
+                self.assert_raises_fpe(overflow,
+                        lambda a,b:a*b, ft_max, ftype(2))
+                self.assert_raises_fpe(overflow,
+                        lambda a,b:a/b, ft_max, ftype(0.5))
+                self.assert_raises_fpe(overflow,
+                        lambda a,b:a+b, ft_max, ft_max*ft_eps)
+                self.assert_raises_fpe(overflow,
+                        lambda a,b:a-b, -ft_max, ft_max*ft_eps)
+                self.assert_raises_fpe(divbyzero,
+                        lambda a,b:a/b, ftype(1), ftype(0))
+                self.assert_raises_fpe(invalid,
+                        lambda a,b:a/b, ftype(np.inf), ftype(np.inf))
+                self.assert_raises_fpe(invalid,
+                        lambda a,b:a/b, ftype(0), ftype(0))
+                self.assert_raises_fpe(invalid,
+                        lambda a,b:a-b, ftype(np.inf), ftype(np.inf))
+                self.assert_raises_fpe(invalid,
+                        lambda a,b:a+b, ftype(np.inf), ftype(-np.inf))
+                self.assert_raises_fpe(invalid,
+                        lambda a,b:a*b, ftype(0), ftype(np.inf))
+        finally:
+            np.seterr(**oldsettings)
+
+    @dec.knownfailureif(True, "See ticket 1755")
+    def test_floating_exceptions_power(self):
+        """Test basic arithmetic function errors"""
+        oldsettings = np.seterr(all='raise')
+        try:
+            # Test for all real and complex float types
+            for typecode in np.typecodes['AllFloat']:
+                ftype = np.obj2sctype(typecode)
+                if np.dtype(ftype).kind == 'f':
+                    # Get some extreme values for the type
+                    fi = np.finfo(ftype)
+                else:
+                    # 'c', complex, corresponding real dtype
+                    rtype = type(ftype(0).real)
+                    fi = np.finfo(rtype)
+                overflow = 'overflow'
+
+                self.assert_raises_fpe(overflow,
+                        np.power, ftype(2), ftype(2**fi.nexp))
+        finally:
+            np.seterr(**oldsettings)
+
+class TestTypes(TestCase):
+    def check_promotion_cases(self, promote_func):
+        """Tests that the scalars get coerced correctly."""
+        b = np.bool_(0)
+        i8, i16, i32, i64 = int8(0), int16(0), int32(0), int64(0)
+        u8, u16, u32, u64 = uint8(0), uint16(0), uint32(0), uint64(0)
+        f32, f64, fld = float32(0), float64(0), longdouble(0)
+        c64, c128, cld = complex64(0), complex128(0), clongdouble(0)
+
+        # coercion within the same kind
+        assert_equal(promote_func(i8,i16), np.dtype(int16))
+        assert_equal(promote_func(i32,i8), np.dtype(int32))
+        assert_equal(promote_func(i16,i64), np.dtype(int64))
+        assert_equal(promote_func(u8,u32), np.dtype(uint32))
+        assert_equal(promote_func(f32,f64), np.dtype(float64))
+        assert_equal(promote_func(fld,f32), np.dtype(longdouble))
+        assert_equal(promote_func(f64,fld), np.dtype(longdouble))
+        assert_equal(promote_func(c128,c64), np.dtype(complex128))
+        assert_equal(promote_func(cld,c128), np.dtype(clongdouble))
+        assert_equal(promote_func(c64,fld), np.dtype(clongdouble))
+
+        # coercion between kinds
+        assert_equal(promote_func(b,i32), np.dtype(int32))
+        assert_equal(promote_func(b,u8), np.dtype(uint8))
+        assert_equal(promote_func(i8,u8), np.dtype(int16))
+        assert_equal(promote_func(u8,i32), np.dtype(int32))
+        assert_equal(promote_func(i64,u32), np.dtype(int64))
+        assert_equal(promote_func(u64,i32), np.dtype(float64))
+        assert_equal(promote_func(i32,f32), np.dtype(float64))
+        assert_equal(promote_func(i64,f32), np.dtype(float64))
+        assert_equal(promote_func(f32,i16), np.dtype(float32))
+        assert_equal(promote_func(f32,u32), np.dtype(float64))
+        assert_equal(promote_func(f32,c64), np.dtype(complex64))
+        assert_equal(promote_func(c128,f32), np.dtype(complex128))
+        assert_equal(promote_func(cld,f64), np.dtype(clongdouble))
+
+        # coercion between scalars and 1-D arrays
+        assert_equal(promote_func(array([b]),i8), np.dtype(int8))
+        assert_equal(promote_func(array([b]),u8), np.dtype(uint8))
+        assert_equal(promote_func(array([b]),i32), np.dtype(int32))
+        assert_equal(promote_func(array([b]),u32), np.dtype(uint32))
+        assert_equal(promote_func(array([i8]),i64), np.dtype(int8))
+        assert_equal(promote_func(u64,array([i32])), np.dtype(int32))
+        assert_equal(promote_func(i64,array([u32])), np.dtype(uint32))
+        assert_equal(promote_func(int32(-1),array([u64])), np.dtype(float64))
+        assert_equal(promote_func(f64,array([f32])), np.dtype(float32))
+        assert_equal(promote_func(fld,array([f32])), np.dtype(float32))
+        assert_equal(promote_func(array([f64]),fld), np.dtype(float64))
+        assert_equal(promote_func(fld,array([c64])), np.dtype(complex64))
+        assert_equal(promote_func(c64,array([f64])), np.dtype(complex128))
+        assert_equal(promote_func(complex64(3j),array([f64])),
+                                                    np.dtype(complex128))
+
+        # coercion between scalars and 1-D arrays, where
+        # the scalar has greater kind than the array
+        assert_equal(promote_func(array([b]),f64), np.dtype(float64))
+        assert_equal(promote_func(array([b]),i64), np.dtype(int64))
+        assert_equal(promote_func(array([b]),u64), np.dtype(uint64))
+        assert_equal(promote_func(array([i8]),f64), np.dtype(float64))
+        assert_equal(promote_func(array([u16]),f64), np.dtype(float64))
+        # uint and int are treated as the same "kind" for
+        # the purposes of array-scalar promotion.
+        assert_equal(promote_func(array([u16]), i32), np.dtype(uint16))
+        # float and complex are treated as the same "kind" for
+        # the purposes of array-scalar promotion, so that you can do
+        # (0j + float32array) to get a complex64 array instead of
+        # a complex128 array.
+        assert_equal(promote_func(array([f32]),c128), np.dtype(complex64))
+
+    def test_coercion(self):
+        def res_type(a, b):
+            return np.add(a, b).dtype
+        self.check_promotion_cases(res_type)
+
+        # Use-case: float/complex scalar * bool/int8 array
+        #           shouldn't narrow the float/complex type
+        for a in [np.array([True,False]), np.array([-3,12], dtype=np.int8)]:
+            b = 1.234 * a
+            assert_equal(b.dtype, np.dtype('f8'), "array type %s" % a.dtype)
+            b = np.longdouble(1.234) * a
+            assert_equal(b.dtype, np.dtype(np.longdouble),
+                                                "array type %s" % a.dtype)
+            b = np.float64(1.234) * a
+            assert_equal(b.dtype, np.dtype('f8'), "array type %s" % a.dtype)
+            b = np.float32(1.234) * a
+            assert_equal(b.dtype, np.dtype('f4'), "array type %s" % a.dtype)
+            b = np.float16(1.234) * a
+            assert_equal(b.dtype, np.dtype('f2'), "array type %s" % a.dtype)
+
+            b = 1.234j * a
+            assert_equal(b.dtype, np.dtype('c16'), "array type %s" % a.dtype)
+            b = np.clongdouble(1.234j) * a
+            assert_equal(b.dtype, np.dtype(np.clongdouble),
+                                                "array type %s" % a.dtype)
+            b = np.complex128(1.234j) * a
+            assert_equal(b.dtype, np.dtype('c16'), "array type %s" % a.dtype)
+            b = np.complex64(1.234j) * a
+            assert_equal(b.dtype, np.dtype('c8'), "array type %s" % a.dtype)
+
+        # The following use-case is problematic, and to resolve its
+        # tricky side-effects requires more changes.
+        #
+        ## Use-case: (1-t)*a, where 't' is a boolean array and 'a' is
+        ##            a float32, shouldn't promote to float64
+        #a = np.array([1.0, 1.5], dtype=np.float32)
+        #t = np.array([True, False])
+        #b = t*a
+        #assert_equal(b, [1.0, 0.0])
+        #assert_equal(b.dtype, np.dtype('f4'))
+        #b = (1-t)*a
+        #assert_equal(b, [0.0, 1.5])
+        #assert_equal(b.dtype, np.dtype('f4'))
+        ## Probably ~t (bitwise negation) is more proper to use here,
+        ## but this is arguably less intuitive to understand at a glance, and
+        ## would fail if 't' is actually an integer array instead of boolean:
+        #b = (~t)*a
+        #assert_equal(b, [0.0, 1.5])
+        #assert_equal(b.dtype, np.dtype('f4'))
+
+    def test_result_type(self):
+        self.check_promotion_cases(np.result_type)
+
+    def test_can_cast(self):
+        assert_(np.can_cast(np.int32, np.int64))
+        assert_(np.can_cast(np.float64, np.complex))
+        assert_(not np.can_cast(np.complex, np.float))
+
+        assert_(np.can_cast('i8', 'f8'))
+        assert_(not np.can_cast('i8', 'f4'))
+        assert_(np.can_cast('i4', 'S4'))
+
+        assert_(np.can_cast('i8', 'i8', 'no'))
+        assert_(not np.can_cast('<i8', '>i8', 'no'))
+
+        assert_(np.can_cast('<i8', '>i8', 'equiv'))
+        assert_(not np.can_cast('<i4', '>i8', 'equiv'))
+
+        assert_(np.can_cast('<i4', '>i8', 'safe'))
+        assert_(not np.can_cast('<i8', '>i4', 'safe'))
+
+        assert_(np.can_cast('<i8', '>i4', 'same_kind'))
+        assert_(not np.can_cast('<i8', '>u4', 'same_kind'))
+
+        assert_(np.can_cast('<i8', '>u4', 'unsafe'))
+
+        assert_raises(TypeError, np.can_cast, 'i4', None)
+        assert_raises(TypeError, np.can_cast, None, 'i4')
+
 class TestFromiter(TestCase):
     def makegen(self):
         for x in xrange(24):
@@ -284,6 +545,49 @@ class TestFromiter(TestCase):
         self.assertTrue(alltrue(a == expected,axis=0))
         self.assertTrue(alltrue(a20 == expected[:20],axis=0))
 
+class TestNonzero(TestCase):
+    def test_nonzero_trivial(self):
+        assert_equal(np.count_nonzero(array([])), 0)
+        assert_equal(np.nonzero(array([])), ([],))
+
+        assert_equal(np.count_nonzero(array(0)), 0)
+        assert_equal(np.nonzero(array(0)), ([],))
+        assert_equal(np.count_nonzero(array(1)), 1)
+        assert_equal(np.nonzero(array(1)), ([0],))
+
+    def test_nonzero_onedim(self):
+        x = array([1,0,2,-1,0,0,8])
+        assert_equal(np.count_nonzero(x), 4)
+        assert_equal(np.nonzero(x), ([0, 2, 3, 6],))
+
+        x = array([(1,2),(0,0),(1,1),(-1,3),(0,7)],
+                            dtype=[('a','i4'),('b','i2')])
+        assert_equal(np.count_nonzero(x['a']), 3)
+        assert_equal(np.count_nonzero(x['b']), 4)
+        assert_equal(np.nonzero(x['a']), ([0,2,3],))
+        assert_equal(np.nonzero(x['b']), ([0,2,3,4],))
+
+    def test_nonzero_twodim(self):
+        x = array([[0,1,0],[2,0,3]])
+        assert_equal(np.count_nonzero(x), 3)
+        assert_equal(np.nonzero(x), ([0,1,1],[1,0,2]))
+
+        x = np.eye(3)
+        assert_equal(np.count_nonzero(x), 3)
+        assert_equal(np.nonzero(x), ([0,1,2],[0,1,2]))
+
+        x = array([[(0,1),(0,0),(1,11)],
+                   [(1,1),(1,0),(0,0)],
+                   [(0,0),(1,5),(0,1)]], dtype=[('a','f4'),('b','u1')])
+        assert_equal(np.count_nonzero(x['a']), 4)
+        assert_equal(np.count_nonzero(x['b']), 5)
+        assert_equal(np.nonzero(x['a']), ([0,1,1,2],[2,0,1,1]))
+        assert_equal(np.nonzero(x['b']), ([0,0,1,2,2],[0,2,0,1,2]))
+
+        assert_equal(np.count_nonzero(x['a'].T), 4)
+        assert_equal(np.count_nonzero(x['b'].T), 5)
+        assert_equal(np.nonzero(x['a'].T), ([0,1,1,2],[1,1,2,0]))
+        assert_equal(np.nonzero(x['b'].T), ([0,0,1,2,2],[0,1,2,0,2]))
 
 class TestIndex(TestCase):
     def test_boolean(self):
@@ -292,7 +596,7 @@ class TestIndex(TestCase):
         g1 = randint(0,5,size=15)
         g2 = randint(0,8,size=15)
         V[g1,g2] = -V[g1,g2]
-        assert (array([a[0][V>0],a[1][V>0],a[2][V>0]]) == a[:,V>0]).all()
+        assert_((array([a[0][V>0],a[1][V>0],a[2][V>0]]) == a[:,V>0]).all())
 
 
 class TestBinaryRepr(TestCase):
@@ -324,55 +628,55 @@ class TestBaseRepr(TestCase):
 class TestArrayComparisons(TestCase):
     def test_array_equal(self):
         res = array_equal(array([1,2]), array([1,2]))
-        assert res
-        assert type(res) is bool
+        assert_(res)
+        assert_(type(res) is bool)
         res = array_equal(array([1,2]), array([1,2,3]))
-        assert not res
-        assert type(res) is bool
+        assert_(not res)
+        assert_(type(res) is bool)
         res = array_equal(array([1,2]), array([3,4]))
-        assert not res
-        assert type(res) is bool
+        assert_(not res)
+        assert_(type(res) is bool)
         res = array_equal(array([1,2]), array([1,3]))
-        assert not res
-        assert type(res) is bool
+        assert_(not res)
+        assert_(type(res) is bool)
 
     def test_array_equiv(self):
         res = array_equiv(array([1,2]), array([1,2]))
-        assert res
-        assert type(res) is bool
+        assert_(res)
+        assert_(type(res) is bool)
         res = array_equiv(array([1,2]), array([1,2,3]))
-        assert not res
-        assert type(res) is bool
+        assert_(not res)
+        assert_(type(res) is bool)
         res = array_equiv(array([1,2]), array([3,4]))
-        assert not res
-        assert type(res) is bool
+        assert_(not res)
+        assert_(type(res) is bool)
         res = array_equiv(array([1,2]), array([1,3]))
-        assert not res
-        assert type(res) is bool
+        assert_(not res)
+        assert_(type(res) is bool)
 
         res = array_equiv(array([1,1]), array([1]))
-        assert res
-        assert type(res) is bool
+        assert_(res)
+        assert_(type(res) is bool)
         res = array_equiv(array([1,1]), array([[1],[1]]))
-        assert res
-        assert type(res) is bool
+        assert_(res)
+        assert_(type(res) is bool)
         res = array_equiv(array([1,2]), array([2]))
-        assert not res
-        assert type(res) is bool
+        assert_(not res)
+        assert_(type(res) is bool)
         res = array_equiv(array([1,2]), array([[1],[2]]))
-        assert not res
-        assert type(res) is bool
+        assert_(not res)
+        assert_(type(res) is bool)
         res = array_equiv(array([1,2]), array([[1,2,3],[4,5,6],[7,8,9]]))
-        assert not res
-        assert type(res) is bool
+        assert_(not res)
+        assert_(type(res) is bool)
 
 
 def assert_array_strict_equal(x, y):
     assert_array_equal(x, y)
     # Check flags
-    assert x.flags == y.flags
+    assert_(x.flags == y.flags)
     # check endianness
-    assert x.dtype.isnative == y.dtype.isnative
+    assert_(x.dtype.isnative == y.dtype.isnative)
 
 
 class TestClip(TestCase):
@@ -412,7 +716,7 @@ class TestClip(TestCase):
     def _generate_non_native_data(self, n, m):
         data = randn(n, m)
         data = self._neg_byteorder(data)
-        assert not data.dtype.isnative
+        assert_(not data.dtype.isnative)
         return data
 
     def _generate_int_data(self, n, m):
@@ -464,7 +768,7 @@ class TestClip(TestCase):
         a   = self._generate_data(self.nr, self.nc)
         m   = -0.5
         M   = self._neg_byteorder(0.6)
-        assert not M.dtype.isnative
+        assert_(not M.dtype.isnative)
         ac  = self.fastclip(a, m, M)
         act = self.clip(a, m, M)
         assert_array_equal(ac, act)
@@ -492,8 +796,8 @@ class TestClip(TestCase):
         """Test clip for non contiguous native input and native scalar min/max."""
         a   = self._generate_data(self.nr * 2, self.nc * 3)
         a   = a[::2, ::3]
-        assert not a.flags['F_CONTIGUOUS']
-        assert not a.flags['C_CONTIGUOUS']
+        assert_(not a.flags['F_CONTIGUOUS'])
+        assert_(not a.flags['C_CONTIGUOUS'])
         ac  = self.fastclip(a, -1.6, 1.7)
         act = self.clip(a, -1.6, 1.7)
         assert_array_strict_equal(ac, act)
@@ -532,7 +836,7 @@ class TestClip(TestCase):
         assert_array_strict_equal(ac, act)
 
     def test_simple_int64_inout(self):
-        """Test native in32 input with double array min/max and int32 out."""
+        """Test native int32 input with double array min/max and int32 out."""
         a   = self._generate_int32_data(self.nr, self.nc)
         m   = zeros(a.shape, float64)
         M   = float64(1)
@@ -577,8 +881,8 @@ class TestClip(TestCase):
         """Test non contiguous double input with double scalar min/max in-place."""
         a   = self._generate_data(self.nr * 2, self.nc * 3)
         a   = a[::2, ::3]
-        assert not a.flags['F_CONTIGUOUS']
-        assert not a.flags['C_CONTIGUOUS']
+        assert_(not a.flags['F_CONTIGUOUS'])
+        assert_(not a.flags['C_CONTIGUOUS'])
         ac  = a.copy()
         m   = -0.5
         M   = 0.6
@@ -648,7 +952,7 @@ class TestClip(TestCase):
         m   = -0.5 * ones(a.shape)
         M   = 1.
         a_s = self._neg_byteorder(a)
-        assert not a_s.dtype.isnative
+        assert_(not a_s.dtype.isnative)
         act = a_s.clip(m, M)
         ac  = self.fastclip(a_s, m, M)
         assert_array_strict_equal(ac, act)
@@ -659,7 +963,7 @@ class TestClip(TestCase):
         m   = -0.5
         M   = 1.
         a_s = self._neg_byteorder(a)
-        assert not a_s.dtype.isnative
+        assert_(not a_s.dtype.isnative)
         ac  = self.fastclip(a_s, m , M)
         act = a_s.clip(m, M)
         assert_array_strict_equal(ac, act)
@@ -670,7 +974,7 @@ class TestClip(TestCase):
         m   = -0.5 * ones(a.shape)
         M   = 1.
         m_s = self._neg_byteorder(m)
-        assert not m_s.dtype.isnative
+        assert_(not m_s.dtype.isnative)
         ac  = self.fastclip(a, m_s , M)
         act = self.clip(a, m_s, M)
         assert_array_strict_equal(ac, act)
@@ -792,18 +1096,18 @@ class TestClip(TestCase):
         a2 = clip(a, m, M, out=a)
         self.clip(a, m, M, ac)
         assert_array_strict_equal(a2, ac)
-        self.assert_(a2 is a)
+        self.assertTrue(a2 is a)
 
 
-class test_allclose_inf(TestCase):
+class TestAllclose(object):
     rtol = 1e-5
     atol = 1e-8
 
     def tst_allclose(self,x,y):
-        assert allclose(x,y), "%s and %s not close" % (x,y)
+        assert_(allclose(x,y), "%s and %s not close" % (x,y))
 
     def tst_not_allclose(self,x,y):
-        assert not allclose(x,y), "%s and %s shouldn't be close" % (x,y)
+        assert_(not allclose(x,y), "%s and %s shouldn't be close" % (x,y))
 
     def test_ip_allclose(self):
         """Parametric test factory."""
@@ -818,7 +1122,9 @@ class test_allclose_inf(TestCase):
                 ([1], [1+rtol+atol]),
                 (arr, arr + arr*rtol),
                 (arr, arr + arr*rtol + atol*2),
-                (aran, aran + aran*rtol),]
+                (aran, aran + aran*rtol),
+                (inf, inf),
+                (inf, [inf])]
 
         for (x,y) in data:
             yield (self.tst_allclose,x,y)
@@ -883,25 +1189,102 @@ class TestStdVarComplex(TestCase):
 
 
 class TestLikeFuncs(TestCase):
-    '''Test zeros_like and empty_like'''
+    '''Test ones_like, zeros_like, and empty_like'''
 
     def setUp(self):
-        self.data = [(array([[1,2,3],[4,5,6]],dtype=int32), (2,3), int32),
-                     (array([[1,2,3],[4,5,6]],dtype=float32), (2,3), float32),
+        self.data = [
+                # Array scalars
+                (array(3.), None),
+                (array(3), 'f8'),
+                # 1D arrays
+                (arange(6, dtype='f4'), None),
+                (arange(6), 'c16'),
+                # 2D C-layout arrays
+                (arange(6).reshape(2,3), None),
+                (arange(6).reshape(3,2), 'i1'),
+                # 2D F-layout arrays
+                (arange(6).reshape((2,3), order='F'), None),
+                (arange(6).reshape((3,2), order='F'), 'i1'),
+                # 3D C-layout arrays
+                (arange(24).reshape(2,3,4), None),
+                (arange(24).reshape(4,3,2), 'f4'),
+                # 3D F-layout arrays
+                (arange(24).reshape((2,3,4), order='F'), None),
+                (arange(24).reshape((4,3,2), order='F'), 'f4'),
+                # 3D non-C/F-layout arrays
+                (arange(24).reshape(2,3,4).swapaxes(0,1), None),
+                (arange(24).reshape(4,3,2).swapaxes(0,1), '?'),
                      ]
 
+    def check_like_function(self, like_function, value):
+        for d, dtype in self.data:
+            # default (K) order, dtype
+            dz = like_function(d, dtype=dtype)
+            assert_equal(dz.shape, d.shape)
+            assert_equal(array(dz.strides)*d.dtype.itemsize,
+                         array(d.strides)*dz.dtype.itemsize)
+            assert_equal(d.flags.c_contiguous, dz.flags.c_contiguous)
+            assert_equal(d.flags.f_contiguous, dz.flags.f_contiguous)
+            if dtype is None:
+                assert_equal(dz.dtype, d.dtype)
+            else:
+                assert_equal(dz.dtype, np.dtype(dtype))
+            if not value is None:
+                assert_(all(dz == value))
+
+            # C order, default dtype
+            dz = like_function(d, order='C', dtype=dtype)
+            assert_equal(dz.shape, d.shape)
+            assert_(dz.flags.c_contiguous)
+            if dtype is None:
+                assert_equal(dz.dtype, d.dtype)
+            else:
+                assert_equal(dz.dtype, np.dtype(dtype))
+            if not value is None:
+                assert_(all(dz == value))
+
+            # F order, default dtype
+            dz = like_function(d, order='F', dtype=dtype)
+            assert_equal(dz.shape, d.shape)
+            assert_(dz.flags.f_contiguous)
+            if dtype is None:
+                assert_equal(dz.dtype, d.dtype)
+            else:
+                assert_equal(dz.dtype, np.dtype(dtype))
+            if not value is None:
+                assert_(all(dz == value))
+
+            # A order
+            dz = like_function(d, order='A', dtype=dtype)
+            assert_equal(dz.shape, d.shape)
+            if d.flags.f_contiguous:
+                assert_(dz.flags.f_contiguous)
+            else:
+                assert_(dz.flags.c_contiguous)
+            if dtype is None:
+                assert_equal(dz.dtype, d.dtype)
+            else:
+                assert_equal(dz.dtype, np.dtype(dtype))
+            if not value is None:
+                assert_(all(dz == value))
+
+        # Test the 'subok' parameter'
+        a = np.matrix([[1,2],[3,4]])
+
+        b = like_function(a)
+        assert_(type(b) is np.matrix)
+
+        b = like_function(a, subok=False)
+        assert_(not (type(b) is np.matrix))
+
+    def test_ones_like(self):
+        self.check_like_function(np.ones_like, 1)
+
     def test_zeros_like(self):
-        for d, dshape, dtype in self.data:
-            dz = zeros_like(d)
-            assert dz.shape == dshape
-            assert dz.dtype.type == dtype
-            assert all(abs(dz) == 0)
+        self.check_like_function(np.zeros_like, 0)
 
     def test_empty_like(self):
-        for d, dshape, dtype in self.data:
-            dz = zeros_like(d)
-            assert dz.shape == dshape
-            assert dz.dtype.type == dtype
+        self.check_like_function(np.empty_like, None)
 
 class _TestCorrelate(TestCase):
     def _setup(self, dt):
