@@ -29,6 +29,23 @@ class TestDivision(TestCase):
         y = x**2/x
         assert_almost_equal(y/x, [1, 1], err_msg=msg)
 
+    def test_zero_division_complex(self):
+        err = np.seterr(invalid="ignore", divide='ignore')
+        try:
+            x = np.array([0.0], dtype=np.complex128)
+            y = 1.0/x
+            assert_(np.isinf(y)[0])
+            y = complex(np.inf, np.nan)/x
+            assert_(np.isinf(y)[0])
+            y = complex(np.nan, np.inf)/x
+            assert_(np.isinf(y)[0])
+            y = complex(np.inf, np.inf)/x
+            assert_(np.isinf(y)[0])
+            y = 0.0/x
+            assert_(np.isnan(y)[0])
+        finally:
+            np.seterr(**err)
+
     def test_floor_division_complex(self):
         # check that implementation is correct
         msg = "Complex floor division implementation check"
@@ -112,6 +129,10 @@ class TestPower(TestCase):
             assert_complex_equal(np.power(zero, -p), cnan)
         assert_complex_equal(np.power(zero, -1+0.2j), cnan)
 
+    def test_fast_power(self):
+        x=np.array([1,2,3], np.int16)
+        assert (x**2.00001).dtype is (x**2.0).dtype
+
 
 class TestLog2(TestCase):
     def test_log2_values(self) :
@@ -171,15 +192,11 @@ class TestLogAddExp2(object):
             np.seterr(**err)
 
     def test_nan(self):
-        err = np.seterr(invalid="ignore")
-        try:
-            assert np.isnan(np.logaddexp2(np.nan, np.inf))
-            assert np.isnan(np.logaddexp2(np.inf, np.nan))
-            assert np.isnan(np.logaddexp2(np.nan, 0))
-            assert np.isnan(np.logaddexp2(0, np.nan))
-            assert np.isnan(np.logaddexp2(np.nan, np.nan))
-        finally:
-            np.seterr(**err)
+        assert np.isnan(np.logaddexp2(np.nan, np.inf))
+        assert np.isnan(np.logaddexp2(np.inf, np.nan))
+        assert np.isnan(np.logaddexp2(np.nan, 0))
+        assert np.isnan(np.logaddexp2(0, np.nan))
+        assert np.isnan(np.logaddexp2(np.nan, np.nan))
 
 
 class TestLog(TestCase):
@@ -241,15 +258,11 @@ class TestLogAddExp(object):
             np.seterr(**err)
 
     def test_nan(self):
-        err = np.seterr(invalid="ignore")
-        try:
-            assert np.isnan(np.logaddexp(np.nan, np.inf))
-            assert np.isnan(np.logaddexp(np.inf, np.nan))
-            assert np.isnan(np.logaddexp(np.nan, 0))
-            assert np.isnan(np.logaddexp(0, np.nan))
-            assert np.isnan(np.logaddexp(np.nan, np.nan))
-        finally:
-            np.seterr(**err)
+        assert np.isnan(np.logaddexp(np.nan, np.inf))
+        assert np.isnan(np.logaddexp(np.inf, np.nan))
+        assert np.isnan(np.logaddexp(np.nan, 0))
+        assert np.isnan(np.logaddexp(0, np.nan))
+        assert np.isnan(np.logaddexp(np.nan, np.nan))
 
 
 class TestLog1p(TestCase):
@@ -422,6 +435,27 @@ class TestLdexp(TestCase):
 
 
 class TestMaximum(TestCase):
+    def test_reduce(self):
+        dflt = np.typecodes['AllFloat']
+        dint = np.typecodes['AllInteger']
+        seq1 = np.arange(11)
+        seq2 = seq1[::-1]
+        func = np.maximum.reduce
+        for dt in dint:
+            tmp1 = seq1.astype(dt)
+            tmp2 = seq2.astype(dt)
+            assert_equal(func(tmp1), 10)
+            assert_equal(func(tmp2), 10)
+        for dt in dflt:
+            tmp1 = seq1.astype(dt)
+            tmp2 = seq2.astype(dt)
+            assert_equal(func(tmp1), 10)
+            assert_equal(func(tmp2), 10)
+            tmp1[::2] = np.nan
+            tmp2[::2] = np.nan
+            assert_equal(func(tmp1), np.nan)
+            assert_equal(func(tmp2), np.nan)
+
     def test_reduce_complex(self):
         assert_equal(np.maximum.reduce([1,2j]),1)
         assert_equal(np.maximum.reduce([1+3j,2j]),1+3j)
@@ -431,26 +465,44 @@ class TestMaximum(TestCase):
         arg1 = np.array([0,   nan, nan])
         arg2 = np.array([nan, 0,   nan])
         out  = np.array([nan, nan, nan])
-        err = np.seterr(invalid="ignore")
-        try:
-            assert_equal(np.maximum(arg1, arg2), out)
-        finally:
-            np.seterr(**err)
+        assert_equal(np.maximum(arg1, arg2), out)
 
     def test_complex_nans(self):
         nan = np.nan
-        err = np.seterr(invalid="ignore")
-        try:
-            for cnan in [complex(nan, 0), complex(0, nan), complex(nan, nan)] :
-                arg1 = np.array([0, cnan, cnan], dtype=np.complex)
-                arg2 = np.array([cnan, 0, cnan], dtype=np.complex)
-                out  = np.array([nan, nan, nan], dtype=np.complex)
-                assert_equal(np.maximum(arg1, arg2), out)
-        finally:
-            np.seterr(**err)
+        for cnan in [complex(nan, 0), complex(0, nan), complex(nan, nan)] :
+            arg1 = np.array([0, cnan, cnan], dtype=np.complex)
+            arg2 = np.array([cnan, 0, cnan], dtype=np.complex)
+            out  = np.array([nan, nan, nan], dtype=np.complex)
+            assert_equal(np.maximum(arg1, arg2), out)
+
+    def test_object_array(self):
+        arg1 = np.arange(5, dtype=np.object)
+        arg2 = arg1 + 1
+        assert_equal(np.maximum(arg1, arg2), arg2)
 
 
 class TestMinimum(TestCase):
+    def test_reduce(self):
+        dflt = np.typecodes['AllFloat']
+        dint = np.typecodes['AllInteger']
+        seq1 = np.arange(11)
+        seq2 = seq1[::-1]
+        func = np.minimum.reduce
+        for dt in dint:
+            tmp1 = seq1.astype(dt)
+            tmp2 = seq2.astype(dt)
+            assert_equal(func(tmp1), 0)
+            assert_equal(func(tmp2), 0)
+        for dt in dflt:
+            tmp1 = seq1.astype(dt)
+            tmp2 = seq2.astype(dt)
+            assert_equal(func(tmp1), 0)
+            assert_equal(func(tmp2), 0)
+            tmp1[::2] = np.nan
+            tmp2[::2] = np.nan
+            assert_equal(func(tmp1), np.nan)
+            assert_equal(func(tmp2), np.nan)
+
     def test_reduce_complex(self):
         assert_equal(np.minimum.reduce([1,2j]),2j)
         assert_equal(np.minimum.reduce([1+3j,2j]),2j)
@@ -460,26 +512,44 @@ class TestMinimum(TestCase):
         arg1 = np.array([0,   nan, nan])
         arg2 = np.array([nan, 0,   nan])
         out  = np.array([nan, nan, nan])
-        err = np.seterr(invalid="ignore")
-        try:
-            assert_equal(np.minimum(arg1, arg2), out)
-        finally:
-            np.seterr(**err)
+        assert_equal(np.minimum(arg1, arg2), out)
 
     def test_complex_nans(self):
         nan = np.nan
-        err = np.seterr(invalid="ignore")
-        try:
-            for cnan in [complex(nan, 0), complex(0, nan), complex(nan, nan)] :
-                arg1 = np.array([0, cnan, cnan], dtype=np.complex)
-                arg2 = np.array([cnan, 0, cnan], dtype=np.complex)
-                out  = np.array([nan, nan, nan], dtype=np.complex)
-                assert_equal(np.minimum(arg1, arg2), out)
-        finally:
-            np.seterr(**err)
+        for cnan in [complex(nan, 0), complex(0, nan), complex(nan, nan)] :
+            arg1 = np.array([0, cnan, cnan], dtype=np.complex)
+            arg2 = np.array([cnan, 0, cnan], dtype=np.complex)
+            out  = np.array([nan, nan, nan], dtype=np.complex)
+            assert_equal(np.minimum(arg1, arg2), out)
+
+    def test_object_array(self):
+        arg1 = np.arange(5, dtype=np.object)
+        arg2 = arg1 + 1
+        assert_equal(np.minimum(arg1, arg2), arg1)
 
 
 class TestFmax(TestCase):
+    def test_reduce(self):
+        dflt = np.typecodes['AllFloat']
+        dint = np.typecodes['AllInteger']
+        seq1 = np.arange(11)
+        seq2 = seq1[::-1]
+        func = np.fmax.reduce
+        for dt in dint:
+            tmp1 = seq1.astype(dt)
+            tmp2 = seq2.astype(dt)
+            assert_equal(func(tmp1), 10)
+            assert_equal(func(tmp2), 10)
+        for dt in dflt:
+            tmp1 = seq1.astype(dt)
+            tmp2 = seq2.astype(dt)
+            assert_equal(func(tmp1), 10)
+            assert_equal(func(tmp2), 10)
+            tmp1[::2] = np.nan
+            tmp2[::2] = np.nan
+            assert_equal(func(tmp1), 9)
+            assert_equal(func(tmp2), 9)
+
     def test_reduce_complex(self):
         assert_equal(np.fmax.reduce([1,2j]),1)
         assert_equal(np.fmax.reduce([1+3j,2j]),1+3j)
@@ -489,26 +559,39 @@ class TestFmax(TestCase):
         arg1 = np.array([0,   nan, nan])
         arg2 = np.array([nan, 0,   nan])
         out  = np.array([0,   0,   nan])
-        err = np.seterr(invalid="ignore")
-        try:
-            assert_equal(np.fmax(arg1, arg2), out)
-        finally:
-            np.seterr(**err)
+        assert_equal(np.fmax(arg1, arg2), out)
 
     def test_complex_nans(self):
         nan = np.nan
-        err = np.seterr(invalid="ignore")
-        try:
-            for cnan in [complex(nan, 0), complex(0, nan), complex(nan, nan)]:
-                arg1 = np.array([0, cnan, cnan], dtype=np.complex)
-                arg2 = np.array([cnan, 0, cnan], dtype=np.complex)
-                out  = np.array([0,    0, nan], dtype=np.complex)
-                assert_equal(np.fmax(arg1, arg2), out)
-        finally:
-            np.seterr(**err)
+        for cnan in [complex(nan, 0), complex(0, nan), complex(nan, nan)] :
+            arg1 = np.array([0, cnan, cnan], dtype=np.complex)
+            arg2 = np.array([cnan, 0, cnan], dtype=np.complex)
+            out  = np.array([0,    0, nan], dtype=np.complex)
+            assert_equal(np.fmax(arg1, arg2), out)
 
 
 class TestFmin(TestCase):
+    def test_reduce(self):
+        dflt = np.typecodes['AllFloat']
+        dint = np.typecodes['AllInteger']
+        seq1 = np.arange(11)
+        seq2 = seq1[::-1]
+        func = np.fmin.reduce
+        for dt in dint:
+            tmp1 = seq1.astype(dt)
+            tmp2 = seq2.astype(dt)
+            assert_equal(func(tmp1), 0)
+            assert_equal(func(tmp2), 0)
+        for dt in dflt:
+            tmp1 = seq1.astype(dt)
+            tmp2 = seq2.astype(dt)
+            assert_equal(func(tmp1), 0)
+            assert_equal(func(tmp2), 0)
+            tmp1[::2] = np.nan
+            tmp2[::2] = np.nan
+            assert_equal(func(tmp1), 1)
+            assert_equal(func(tmp2), 1)
+
     def test_reduce_complex(self):
         assert_equal(np.fmin.reduce([1,2j]),2j)
         assert_equal(np.fmin.reduce([1+3j,2j]),2j)
@@ -518,23 +601,15 @@ class TestFmin(TestCase):
         arg1 = np.array([0,   nan, nan])
         arg2 = np.array([nan, 0,   nan])
         out  = np.array([0,   0,   nan])
-        err = np.seterr(invalid="ignore")
-        try:
-            assert_equal(np.fmin(arg1, arg2), out)
-        finally:
-            np.seterr(**err)
+        assert_equal(np.fmin(arg1, arg2), out)
 
     def test_complex_nans(self):
         nan = np.nan
-        err = np.seterr(invalid="ignore")
-        try:
-            for cnan in [complex(nan, 0), complex(0, nan), complex(nan, nan)]:
-                arg1 = np.array([0, cnan, cnan], dtype=np.complex)
-                arg2 = np.array([cnan, 0, cnan], dtype=np.complex)
-                out  = np.array([0,    0, nan], dtype=np.complex)
-                assert_equal(np.fmin(arg1, arg2), out)
-        finally:
-            np.seterr(**err)
+        for cnan in [complex(nan, 0), complex(0, nan), complex(nan, nan)] :
+            arg1 = np.array([0, cnan, cnan], dtype=np.complex)
+            arg2 = np.array([cnan, 0, cnan], dtype=np.complex)
+            out  = np.array([0,    0, nan], dtype=np.complex)
+            assert_equal(np.fmin(arg1, arg2), out)
 
 
 class TestFloatingPoint(TestCase):
@@ -1038,8 +1113,8 @@ def test_nextafterf():
     return _test_nextafter(np.float32)
 
 @dec.knownfailureif(sys.platform == 'win32' or
-                    (sys.platform == "darwin" and "powerpc" in platform.processor()),
-                    "Long double support buggy on win32 and OS X on PPC.")
+                    ("powerpc" in platform.processor()),
+                    "Long double support buggy on win32 and PPC.")
 def test_nextafterl():
     return _test_nextafter(np.longdouble)
 
@@ -1065,8 +1140,8 @@ def test_spacingf():
     return _test_spacing(np.float32)
 
 @dec.knownfailureif(sys.platform == 'win32' or
-                    (sys.platform == "darwin" and "powerpc" in platform.processor()),
-                    "Long double support buggy on win32 and OS X on PPC.")
+                    ("powerpc" in platform.processor()),
+                    "Long double support buggy on win32 and PPC.")
 def test_spacingl():
     return _test_spacing(np.longdouble)
 
@@ -1148,23 +1223,20 @@ def test_complex_nan_comparisons():
     fins = [complex(1, 0), complex(-1, 0), complex(0, 1), complex(0, -1),
             complex(1, 1), complex(-1, -1), complex(0, 0)]
 
-    err = np.seterr(invalid="ignore")
-    try:
-        for x in nans + fins:
-            x = np.array([x])
-            for y in nans + fins:
-                y = np.array([y])
-    
-                if np.isfinite(x) and np.isfinite(y):
-                    continue
-    
-                assert_equal(x < y, False, err_msg="%r < %r" % (x, y))
-                assert_equal(x > y, False, err_msg="%r > %r" % (x, y))
-                assert_equal(x <= y, False, err_msg="%r <= %r" % (x, y))
-                assert_equal(x >= y, False, err_msg="%r >= %r" % (x, y))
-                assert_equal(x == y, False, err_msg="%r == %r" % (x, y))
-    finally:
-        np.seterr(**err)
+    for x in nans + fins:
+        x = np.array([x])
+        for y in nans + fins:
+            y = np.array([y])
+
+            if np.isfinite(x) and np.isfinite(y):
+                continue
+
+            assert_equal(x < y, False, err_msg="%r < %r" % (x, y))
+            assert_equal(x > y, False, err_msg="%r > %r" % (x, y))
+            assert_equal(x <= y, False, err_msg="%r <= %r" % (x, y))
+            assert_equal(x >= y, False, err_msg="%r >= %r" % (x, y))
+            assert_equal(x == y, False, err_msg="%r == %r" % (x, y))
+
 
 if __name__ == "__main__":
     run_module_suite()
