@@ -186,8 +186,8 @@ class build_src(build_ext.build_ext):
                     build_dir = self.get_package_dir('.'.join(d.split(os.sep)))
                 else:
                     build_dir = os.path.join(self.build_src,d)
-                funcs = filter(callable,files)
-                files = filter(lambda f:not callable(f), files)
+                funcs = filter(lambda f:hasattr(f, '__call__'), files)
+                files = filter(lambda f:not hasattr(f, '__call__'), files)
                 for f in funcs:
                     if f.func_code.co_argcount==1:
                         s = f(build_dir)
@@ -274,7 +274,7 @@ class build_src(build_ext.build_ext):
                 else:
                     build_dir = os.path.join(self.build_src,
                                              os.path.join(*package.split('.')))
-                if callable(source):
+                if hasattr(source, '__call__'):
                     target = os.path.join(build_dir, module_base + '.py')
                     source = source(target)
                 if source is None:
@@ -481,8 +481,8 @@ class build_src(build_ext.build_ext):
                 pyrex_result = Pyrex.Compiler.Main.compile(source,
                                                            options=options)
                 if pyrex_result.num_errors != 0:
-                    raise DistutilsError,"%d errors while compiling %r with Pyrex" \
-                          % (pyrex_result.num_errors, source)
+                    raise DistutilsError("%d errors while compiling %r with Pyrex" \
+                          % (pyrex_result.num_errors, source))
             elif os.path.isfile(target_file):
                 log.warn("Pyrex required for compiling %r but not available,"\
                          " using old target %r"\
@@ -640,7 +640,11 @@ class build_src(build_ext.build_ext):
         target_dirs = []
         py_files = []     # swig generated .py files
         target_ext = '.c'
-        if self.swig_cpp:
+        if '-c++' in extension.swig_opts:
+            typ = 'c++'
+            is_cpp = True
+            extension.swig_opts.remove('-c++')
+        elif self.swig_cpp:
             typ = 'c++'
             is_cpp = True
         else:
@@ -652,6 +656,8 @@ class build_src(build_ext.build_ext):
         for source in sources:
             (base, ext) = os.path.splitext(source)
             if ext == '.i': # SWIG interface file
+                # the code below assumes that the sources list
+                # contains not more than one .i SWIG interface file
                 if self.inplace:
                     target_dir = os.path.dirname(base)
                     py_target_dir = self.ext_target_dir
@@ -667,10 +673,16 @@ class build_src(build_ext.build_ext):
                     if typ is None:
                         typ = get_swig_target(source)
                         is_cpp = typ=='c++'
-                        if is_cpp: target_ext = '.cpp'
+                        if is_cpp: 
+                            target_ext = '.cpp'
                     else:
                         typ2 = get_swig_target(source)
-                        if typ!=typ2:
+                        if typ2 is None:
+                            log.warn('source %r does not define swig target, assuming %s swig target' \
+                                     % (source, typ))
+                            if is_cpp: 
+                                target_ext = '.cpp'
+                        elif typ!=typ2:
                             log.warn('expected %r but source %r defines %r swig target' \
                                      % (typ, source, typ2))
                             if typ2=='c++':
@@ -716,7 +728,7 @@ class build_src(build_ext.build_ext):
             self.mkpath(d)
 
         swig = self.swig or self.find_swig()
-        swig_cmd = [swig, "-python"]
+        swig_cmd = [swig, "-python"] + extension.swig_opts
         if is_cpp:
             swig_cmd.append('-c++')
         for d in extension.include_dirs:
@@ -746,7 +758,7 @@ _has_cpp_header = re.compile(r'-[*]-\s*c[+][+]\s*-[*]-',re.I).search
 
 def get_swig_target(source):
     f = open(source,'r')
-    result = 'c'
+    result = None
     line = f.readline()
     if _has_cpp_header(line):
         result = 'c++'
