@@ -13,13 +13,13 @@ class TestAny(TestCase):
         y1 = [0, 0, 1, 0]
         y2 = [0, 0, 0, 0]
         y3 = [1, 0, 1, 0]
-        assert(any(y1))
-        assert(any(y3))
-        assert(not any(y2))
+        assert_(any(y1))
+        assert_(any(y3))
+        assert_(not any(y2))
 
     def test_nd(self):
         y1 = [[0, 0, 0], [0, 1, 0], [1, 1, 0]]
-        assert(any(y1))
+        assert_(any(y1))
         assert_array_equal(sometrue(y1, axis=0), [1, 1, 0])
         assert_array_equal(sometrue(y1, axis=1), [0, 1, 1])
 
@@ -29,14 +29,14 @@ class TestAll(TestCase):
         y1 = [0, 1, 1, 0]
         y2 = [0, 0, 0, 0]
         y3 = [1, 1, 1, 1]
-        assert(not all(y1))
-        assert(all(y3))
-        assert(not all(y2))
-        assert(all(~array(y2)))
+        assert_(not all(y1))
+        assert_(all(y3))
+        assert_(not all(y2))
+        assert_(all(~array(y2)))
 
     def test_nd(self):
         y1 = [[0, 0, 1], [0, 1, 1], [1, 1, 1]]
-        assert(not all(y1))
+        assert_(not all(y1))
         assert_array_equal(alltrue(y1, axis=0), [0, 0, 1])
         assert_array_equal(alltrue(y1, axis=1), [0, 0, 1])
 
@@ -44,11 +44,11 @@ class TestAll(TestCase):
 class TestAverage(TestCase):
     def test_basic(self):
         y1 = array([1, 2, 3])
-        assert(average(y1, axis=0) == 2.)
+        assert_(average(y1, axis=0) == 2.)
         y2 = array([1., 2., 3.])
-        assert(average(y2, axis=0) == 2.)
+        assert_(average(y2, axis=0) == 2.)
         y3 = [0., 0., 0.]
-        assert(average(y3, axis=0) == 0.)
+        assert_(average(y3, axis=0) == 0.)
 
         y4 = ones((4, 4))
         y4[0, 1] = 0
@@ -407,7 +407,7 @@ class TestDigitize(TestCase):
     def test_random(self):
         x = rand(10)
         bin = linspace(x.min(), x.max(), 10)
-        assert all(digitize(x, bin) != 0)
+        assert_(all(digitize(x, bin) != 0))
 
 
 class TestUnwrap(TestCase):
@@ -415,7 +415,7 @@ class TestUnwrap(TestCase):
                 #check that unwrap removes jumps greather that 2*pi
         assert_array_equal(unwrap([1, 1 + 2 * pi]), [1, 1])
         #check that unwrap maintans continuity
-        assert(all(diff(unwrap(rand(10) * 100)) < pi))
+        assert_(all(diff(unwrap(rand(10) * 100)) < pi))
 
 
 class TestFilterwindows(TestCase):
@@ -520,7 +520,7 @@ class TestTrapz(TestCase):
 
 class TestSinc(TestCase):
     def test_simple(self):
-        assert(sinc(0) == 1)
+        assert_(sinc(0) == 1)
         w = sinc(linspace(-1, 1, 100))
         #check symmetry
         assert_array_almost_equal(w, flipud(w), 7)
@@ -556,6 +556,10 @@ class TestHistogram(TestCase):
         hist, edges = histogram([1, 2, 3, 4], [1, 2])
         assert_array_equal(hist, [2, ])
         assert_array_equal(edges, [1, 2])
+        assert_raises(ValueError, histogram, [1, 2], bins=0)
+        h, e = histogram([1,2], bins=1)
+        assert_equal(h, array([2]))
+        assert_allclose(e, array([1., 2.]))
 
     def test_normed(self):
         # Check that the integral of the density equals 1.
@@ -565,13 +569,40 @@ class TestHistogram(TestCase):
         area = sum(a * diff(b))
         assert_almost_equal(area, 1)
 
-        # Check with non constant bin width
-        v = rand(n) * 10
+        # Check with non-constant bin widths (buggy but backwards compatible)
+        v = np.arange(10)
         bins = [0, 1, 5, 9, 10]
         a, b = histogram(v, bins, normed=True)
         area = sum(a * diff(b))
         assert_almost_equal(area, 1)
 
+    def test_density(self):
+        # Check that the integral of the density equals 1.
+        n = 100
+        v = rand(n)
+        a, b = histogram(v, density=True)
+        area = sum(a * diff(b))
+        assert_almost_equal(area, 1)
+
+        # Check with non-constant bin widths
+        v = np.arange(10)
+        bins = [0,1,3,6,10]
+        a, b = histogram(v, bins, density=True)
+        assert_array_equal(a, .1)
+        assert_equal(sum(a*diff(b)), 1)
+
+        # Variale bin widths are especially useful to deal with
+        # infinities.
+        v = np.arange(10)
+        bins = [0,1,3,6,np.inf]
+        a, b = histogram(v, bins, density=True)
+        assert_array_equal(a, [.1,.1,.1,0.])
+
+        # Taken from a bug report from N. Becker on the numpy-discussion
+        # mailing list Aug. 6, 2010.
+        counts, dmy = np.histogram([1,2,3,4], [0.5,1.5,np.inf], density=True)
+        assert_equal(counts, [.25, 0])
+
     def test_outliers(self):
         # Check that outliers are not tallied
         a = arange(10) + .5
@@ -600,16 +631,16 @@ class TestHistogram(TestCase):
         # Check the type of the returned histogram
         a = arange(10) + .5
         h, b = histogram(a)
-        assert(issubdtype(h.dtype, int))
+        assert_(issubdtype(h.dtype, int))
 
         h, b = histogram(a, normed=True)
-        assert(issubdtype(h.dtype, float))
+        assert_(issubdtype(h.dtype, float))
 
         h, b = histogram(a, weights=ones(10, int))
-        assert(issubdtype(h.dtype, int))
+        assert_(issubdtype(h.dtype, int))
 
         h, b = histogram(a, weights=ones(10, float))
-        assert(issubdtype(h.dtype, float))
+        assert_(issubdtype(h.dtype, float))
 
     def test_weights(self):
         v = rand(100)
@@ -631,7 +662,17 @@ class TestHistogram(TestCase):
         wa, wb = histogram([1, 2, 2, 4], bins=4, weights=[4, 3, 2, 1])
         assert_array_equal(wa, [4, 5, 0, 1])
         wa, wb = histogram([1, 2, 2, 4], bins=4, weights=[4, 3, 2, 1], normed=True)
-        assert_array_equal(wa, array([4, 5, 0, 1]) / 10. / 3. * 4)
+        assert_array_almost_equal(wa, array([4, 5, 0, 1]) / 10. / 3. * 4)
+
+        # Check weights with non-uniform bin widths
+        a,b = histogram(np.arange(9), [0,1,3,6,10], \
+                        weights=[2,1,1,1,1,1,1,1,1], density=True)
+        assert_almost_equal(a, [.2, .1, .1, .075])
+
+    def test_empty(self):
+        a, b = histogram([], bins=([0,1]))
+        assert_array_equal(a, array([0]))
+        assert_array_equal(b, array([0, 1]))
 
 
 class TestHistogramdd(TestCase):
@@ -645,7 +686,7 @@ class TestHistogramdd(TestCase):
         # Check normalization
         ed = [[-2, 0, 2], [0, 1, 2, 3], [0, 1, 2, 3]]
         H, edges = histogramdd(x, bins=ed, normed=True)
-        assert(all(H == answer / 12.))
+        assert_(all(H == answer / 12.))
         # Check that H has the correct shape.
         H, edges = histogramdd(x, (2, 3, 4), range=[[-1, 1], [0, 3], [0, 4]],
             normed=True)
@@ -674,7 +715,7 @@ class TestHistogramdd(TestCase):
         r = rand(10, 3)
         for b in bins:
             H, edges = histogramdd(r, b)
-            assert(H.shape == b)
+            assert_(H.shape == b)
 
     def test_shape_4d(self):
         # All possible permutations for bins of different lengths in 4D.
@@ -688,7 +729,7 @@ class TestHistogramdd(TestCase):
         r = rand(10, 4)
         for b in bins:
             H, edges = histogramdd(r, b)
-            assert(H.shape == b)
+            assert_(H.shape == b)
 
     def test_weights(self):
         v = rand(100, 2)
@@ -706,16 +747,44 @@ class TestHistogramdd(TestCase):
         hist, edges = histogramdd(x, bins=2)
         assert_array_equal(edges[0], array([-0.5, 0. , 0.5]))
 
+    def test_empty(self):
+        a, b = histogramdd([[], []], bins=([0,1], [0,1]))
+        assert_array_max_ulp(a, array([[ 0.]]))
+        a, b = np.histogramdd([[], [], []], bins=2)
+        assert_array_max_ulp(a, np.zeros((2, 2, 2)))
+
+
+    def test_bins_errors(self):
+        """There are two ways to specify bins. Check for the right errors when
+        mixing those."""
+        x = np.arange(8).reshape(2, 4)
+        assert_raises(ValueError, np.histogramdd, x, bins=[-1, 2, 4, 5])
+        assert_raises(ValueError, np.histogramdd, x, bins=[1, 0.99, 1, 1])
+        assert_raises(ValueError, np.histogramdd, x, bins=[1, 1, 1, [1, 2, 2, 3]])
+        assert_raises(ValueError, np.histogramdd, x, bins=[1, 1, 1, [1, 2, 3, -3]])
+        assert_(np.histogramdd(x, bins=[1, 1, 1, [1, 2, 3, 4]]))
+
+    def test_inf_edges(self):
+        """Test using +/-inf bin edges works. See #1788."""
+        x = np.arange(6).reshape(3, 2)
+        expected = np.array([[1, 0], [0, 1], [0, 1]])
+        h, e = np.histogramdd(x, bins=[3, [-np.inf, 2, 10]])
+        assert_allclose(h, expected)
+        h, e = np.histogramdd(x, bins=[3, np.array([-1, 2, np.inf])])
+        assert_allclose(h, expected)
+        h, e = np.histogramdd(x, bins=[3, [-np.inf, 3, np.inf]])
+        assert_allclose(h, expected)
+
 
 class TestUnique(TestCase):
     def test_simple(self):
         x = array([4, 3, 2, 1, 1, 2, 3, 4, 0])
-        assert(all(unique(x) == [0, 1, 2, 3, 4]))
-        assert(unique(array([1, 1, 1, 1, 1])) == array([1]))
+        assert_(all(unique(x) == [0, 1, 2, 3, 4]))
+        assert_(unique(array([1, 1, 1, 1, 1])) == array([1]))
         x = ['widget', 'ham', 'foo', 'bar', 'foo', 'ham']
-        assert(all(unique(x) == ['bar', 'foo', 'ham', 'widget']))
+        assert_(all(unique(x) == ['bar', 'foo', 'ham', 'widget']))
         x = array([5 + 6j, 1 + 1j, 1 + 10j, 10, 5 + 6j])
-        assert(all(unique(x) == [1 + 1j, 1 + 10j, 5 + 6j, 10]))
+        assert_(all(unique(x) == [1 + 1j, 1 + 10j, 5 + 6j, 10]))
 
 
 class TestCheckFinite(TestCase):
@@ -769,7 +838,7 @@ class TestNaNFuncts(TestCase):
                             array([[ 0.01319214, 0.11704017, 0.1630199 ],
                                    [ 0.37910852, 0.87964135, 0.34306596],
                                    [ 0.72687499, 0.23913896, 0.33850425]]))
-        assert nanmin([nan, nan]) is nan
+        assert_(np.isnan(nanmin([nan, nan])))
 
     def test_nanargmin(self):
         assert_almost_equal(nanargmin(self.A), 1)
@@ -800,6 +869,7 @@ class TestNaNFuncts(TestCase):
                             array([[ 0.01620964, 0.75157887, 0.28333658],
                                    [ 0.59541557, 0.87964135, 0.70543747],
                                    [ 0.91084584, 0.84386844, 0.37068164]]))
+        assert_(np.isnan(nanmax([nan, nan])))
 
     def test_nanmin_allnan_on_axis(self):
         assert_array_equal(isnan(nanmin([[nan] * 2] * 3, axis=1)),
@@ -877,6 +947,20 @@ class TestCorrCoef(TestCase):
         assert_almost_equal(corrcoef(self.A, ddof=-1), self.res1)
         assert_almost_equal(corrcoef(self.A, self.B, ddof=-1), self.res2)
 
+    def test_empty(self):
+        assert_equal(corrcoef(np.array([])).size, 0)
+        assert_equal(corrcoef(np.array([]).reshape(0, 2)).shape, (0, 2))
+
+
+class TestCov(TestCase):
+    def test_basic(self):
+        x = np.array([[0, 2], [1, 1], [2, 0]]).T
+        assert_allclose(np.cov(x), np.array([[ 1.,-1.], [-1.,1.]]))
+
+    def test_empty(self):
+        assert_equal(cov(np.array([])).size, 0)
+        assert_equal(cov(np.array([]).reshape(0, 2)).shape, (0, 2))
+
 
 class Test_i0(TestCase):
     def test_simple(self):
@@ -900,7 +984,7 @@ class Test_i0(TestCase):
 class TestKaiser(TestCase):
     def test_simple(self):
         assert_almost_equal(kaiser(0, 1.0), array([]))
-        assert isfinite(kaiser(1, 1.0))
+        assert_(isfinite(kaiser(1, 1.0)))
         assert_almost_equal(kaiser(2, 1.0), array([ 0.78984831, 0.78984831]))
         assert_almost_equal(kaiser(5, 1.0),
                             array([ 0.78984831, 0.94503323, 1.        ,
@@ -927,14 +1011,14 @@ class TestMsort(TestCase):
 class TestMeshgrid(TestCase):
     def test_simple(self):
         [X, Y] = meshgrid([1, 2, 3], [4, 5, 6, 7])
-        assert all(X == array([[1, 2, 3],
+        assert_(all(X == array([[1, 2, 3],
                                [1, 2, 3],
                                [1, 2, 3],
-                               [1, 2, 3]]))
-        assert all(Y == array([[4, 4, 4],
+                               [1, 2, 3]])))
+        assert_(all(Y == array([[4, 4, 4],
                                [5, 5, 5],
                                [6, 6, 6],
-                               [7, 7, 7]]))
+                               [7, 7, 7]])))
 
 
 class TestPiecewise(TestCase):
@@ -978,8 +1062,8 @@ class TestPiecewise(TestCase):
     def test_0d(self):
         x = array(3)
         y = piecewise(x, x > 3, [4, 0])
-        assert y.ndim == 0
-        assert y == 0
+        assert_(y.ndim == 0)
+        assert_(y == 0)
 
 
 class TestBincount(TestCase):
@@ -1003,6 +1087,32 @@ class TestBincount(TestCase):
         y = np.bincount(x, w)
         assert_array_equal(y, np.array([0, 0.2, 0.5, 0, 0.5, 0.1]))
 
+    def test_with_minlength(self):
+        x = np.array([0, 1, 0, 1, 1])
+        y = np.bincount(x, minlength=3)
+        assert_array_equal(y, np.array([2, 3, 0]))
+
+    def test_with_minlength_smaller_than_maxvalue(self):
+        x = np.array([0, 1, 1, 2, 2, 3, 3])
+        y = np.bincount(x, minlength=2)
+        assert_array_equal(y, np.array([1, 2, 2, 2]))
+
+    def test_with_minlength_and_weights(self):
+        x = np.array([1, 2, 4, 5, 2])
+        w = np.array([0.2, 0.3, 0.5, 0.1, 0.2])
+        y = np.bincount(x, w, 8)
+        assert_array_equal(y, np.array([0, 0.2, 0.5, 0, 0.5, 0.1, 0, 0]))
+
+    def test_empty(self):
+        x = np.array([], dtype=int)
+        y = np.bincount(x)
+        assert_array_equal(x,y)
+
+    def test_empty_with_minlength(self):
+        x = np.array([], dtype=int)
+        y = np.bincount(x, minlength=5)
+        assert_array_equal(y, np.zeros(5, dtype=int))
+
 
 class TestInterp(TestCase):
     def test_exceptions(self):
@@ -1069,5 +1179,16 @@ def test_percentile_out():
     assert_equal(y, np.percentile(x, p, axis=1))
 
 
+def test_median():
+    a0 = np.array(1)
+    a1 = np.arange(2)
+    a2 = np.arange(6).reshape(2, 3)
+    assert_allclose(np.median(a0), 1)
+    assert_allclose(np.median(a1), 0.5)
+    assert_allclose(np.median(a2), 2.5)
+    assert_allclose(np.median(a2, axis=0), [1.5,  2.5,  3.5])
+    assert_allclose(np.median(a2, axis=1), [1, 4])
+
+
 if __name__ == "__main__":
     run_module_suite()
