@@ -1,7 +1,8 @@
 import numpy as np
 import numpy.ma as ma
-from numpy.ma.testutils import *
-from numpy.testing import assert_warns
+from numpy.ma.testutils import (TestCase, assert_equal, assert_array_equal,
+    assert_raises, run_module_suite)
+from numpy.testing import assert_warns, assert_, build_err_msg
 
 import sys
 
@@ -12,6 +13,9 @@ import threading
 from tempfile import mkstemp, NamedTemporaryFile
 import time
 from datetime import datetime
+import warnings
+import gc
+from numpy.testing.utils import WarningManager
 
 from numpy.lib._iotools import ConverterError, ConverterLockError, \
                                ConversionWarning
@@ -237,6 +241,58 @@ class TestSaveTxt(TestCase):
         finally:
             os.unlink(name)
 
+    def test_complex_arrays(self):
+        ncols = 2
+        nrows = 2
+        a = np.zeros((ncols, nrows), dtype=np.complex128)
+        re = np.pi
+        im = np.e
+        a[:] = re + 1.0j * im
+        # One format only
+        c = StringIO()
+        np.savetxt(c, a, fmt=' %+.3e')
+        c.seek(0)
+        lines = c.readlines()
+        _assert_floatstr_lines_equal(lines, asbytes_nested([
+            ' ( +3.142e+00+ +2.718e+00j)  ( +3.142e+00+ +2.718e+00j)\n',
+            ' ( +3.142e+00+ +2.718e+00j)  ( +3.142e+00+ +2.718e+00j)\n']))
+        # One format for each real and imaginary part
+        c = StringIO()
+        np.savetxt(c, a, fmt='  %+.3e' * 2 * ncols)
+        c.seek(0)
+        lines = c.readlines()
+        _assert_floatstr_lines_equal(lines, asbytes_nested([
+            '  +3.142e+00  +2.718e+00  +3.142e+00  +2.718e+00\n',
+            '  +3.142e+00  +2.718e+00  +3.142e+00  +2.718e+00\n']))
+        # One format for each complex number
+        c = StringIO()
+        np.savetxt(c, a, fmt=['(%.3e%+.3ej)'] * ncols)
+        c.seek(0)
+        lines = c.readlines()
+        _assert_floatstr_lines_equal(lines, asbytes_nested([
+            '(3.142e+00+2.718e+00j) (3.142e+00+2.718e+00j)\n',
+            '(3.142e+00+2.718e+00j) (3.142e+00+2.718e+00j)\n']))
+
+
+def _assert_floatstr_lines_equal(actual_lines, expected_lines):
+    """A string comparison function that also works on Windows + Python 2.5.
+
+    This is necessary because Python 2.5 on Windows inserts an extra 0 in
+    the exponent of the string representation of floating point numbers.
+
+    Only used in TestSaveTxt.test_complex_arrays, no attempt made to make this
+    more generic.
+
+    Once Python 2.5 compatibility is dropped, simply use `assert_equal` instead
+    of this function.
+    """
+    for actual, expected in zip(actual_lines, expected_lines):
+        if actual != expected:
+            expected_win25 = expected.replace("e+00", "e+000")
+            if actual != expected_win25:
+                msg = build_err_msg([actual, expected], '', verbose=True)
+                raise AssertionError(msg)
+
 
 class TestLoadTxt(TestCase):
     def test_record(self):
@@ -381,9 +437,24 @@ class TestLoadTxt(TestCase):
                      dtype=dt)
         assert_array_equal(x, a)
 
+    def test_3d_shaped_dtype(self):
+        c = StringIO("aaaa  1.0  8.0  1 2 3 4 5 6 7 8 9 10 11 12")
+        dt = np.dtype([('name', 'S4'), ('x', float), ('y', float),
+                       ('block', int, (2, 2, 3))])
+        x = np.loadtxt(c, dtype=dt)
+        a = np.array([('aaaa', 1.0, 8.0, [[[1, 2, 3], [4, 5, 6]],[[7, 8, 9], [10, 11, 12]]])],
+                     dtype=dt)
+        assert_array_equal(x, a)
+
     def test_empty_file(self):
+        warnings.filterwarnings("ignore", message="loadtxt: Empty input file:")
         c = StringIO()
-        assert_raises(IOError, np.loadtxt, c)
+        x = np.loadtxt(c)
+        assert_equal(x.shape, (0,))
+        x = np.loadtxt(c, dtype=np.int64)
+        assert_equal(x.shape, (0,))
+        assert_(x.dtype == np.int64)
+
 
     def test_unused_converter(self):
         c = StringIO()
@@ -402,10 +473,8 @@ class TestLoadTxt(TestCase):
         "Test using an explicit dtype with an object"
         from datetime import date
         import time
-        data = """
-        1; 2001-01-01
-        2; 2002-01-31
-        """
+        data = asbytes(""" 1; 2001-01-01
+                           2; 2002-01-31 """)
         ndtype = [('idx', int), ('code', np.object)]
         func = lambda s: strptime(s.strip(), "%Y-%m-%d")
         converters = {1: func}
@@ -415,6 +484,22 @@ class TestLoadTxt(TestCase):
                            dtype=ndtype)
         assert_equal(test, control)
 
+    def test_uint64_type(self):
+        tgt = (9223372043271415339, 9223372043271415853)
+        c = StringIO()
+        c.write(asbytes("%s %s" % tgt))
+        c.seek(0)
+        res = np.loadtxt(c, dtype=np.uint64)
+        assert_equal(res, tgt)
+
+    def test_int64_type(self):
+        tgt = (-9223372036854775807, 9223372036854775807)
+        c = StringIO()
+        c.write(asbytes("%s %s" % tgt))
+        c.seek(0)
+        res = np.loadtxt(c, dtype=np.int64)
+        assert_equal(res, tgt)
+
     def test_universal_newline(self):
         f, name = mkstemp()
         os.write(f, asbytes('1 21\r3 42\r'))
@@ -426,6 +511,71 @@ class TestLoadTxt(TestCase):
         finally:
             os.unlink(name)
 
+    def test_empty_field_after_tab(self):
+        c = StringIO()
+        c.write(asbytes('1 \t2 \t3\tstart \n4\t5\t6\t  \n7\t8\t9.5\t'))
+        c.seek(0)
+        dt = { 'names': ('x', 'y', 'z', 'comment'),
+               'formats': ('<i4', '<i4', '<f4', '|S8')}
+        x = np.loadtxt(c, dtype=dt, delimiter='\t')
+        a = np.array([asbytes('start '), asbytes('  '), asbytes('')])
+        assert_array_equal(x['comment'], a)
+
+    def test_structure_unpack(self):
+        txt = StringIO(asbytes("M 21 72\nF 35 58"))
+        dt = { 'names': ('a', 'b', 'c'), 'formats': ('|S1', '<i4', '<f4')}
+        a, b, c = np.loadtxt(txt, dtype=dt, unpack=True)
+        assert_(a.dtype.str == '|S1')
+        assert_(b.dtype.str == '<i4')
+        assert_(c.dtype.str == '<f4')
+        assert_array_equal(a, np.array([asbytes('M'), asbytes('F')]))
+        assert_array_equal(b, np.array([21, 35]))
+        assert_array_equal(c, np.array([ 72.,  58.]))
+
+    def test_ndmin_keyword(self):
+        c = StringIO()
+        c.write(asbytes('1,2,3\n4,5,6'))
+        c.seek(0)
+        assert_raises(ValueError, np.loadtxt, c, ndmin=3)
+        c.seek(0)
+        assert_raises(ValueError, np.loadtxt, c, ndmin=1.5)
+        c.seek(0)
+        x = np.loadtxt(c, dtype=int, delimiter=',', ndmin=1)
+        a = np.array([[1, 2, 3], [4, 5, 6]])
+        assert_array_equal(x, a)
+        d = StringIO()
+        d.write(asbytes('0,1,2'))
+        d.seek(0)
+        x = np.loadtxt(d, dtype=int, delimiter=',', ndmin=2)
+        assert_(x.shape == (1, 3))
+        d.seek(0)
+        x = np.loadtxt(d, dtype=int, delimiter=',', ndmin=1)
+        assert_(x.shape == (3,))
+        d.seek(0)
+        x = np.loadtxt(d, dtype=int, delimiter=',', ndmin=0)
+        assert_(x.shape == (3,))
+        e = StringIO()
+        e.write(asbytes('0\n1\n2'))
+        e.seek(0)
+        x = np.loadtxt(e, dtype=int, delimiter=',', ndmin=2)
+        assert_(x.shape == (3, 1))
+        e.seek(0)
+        x = np.loadtxt(e, dtype=int, delimiter=',', ndmin=1)
+        assert_(x.shape == (3,))
+        e.seek(0)
+        x = np.loadtxt(e, dtype=int, delimiter=',', ndmin=0)
+        assert_(x.shape == (3,))
+        f = StringIO()
+        assert_(np.loadtxt(f, ndmin=2).shape == (0, 1,))
+        assert_(np.loadtxt(f, ndmin=1).shape == (0,))
+
+    def test_generator_source(self):
+        def count():
+            for i in range(10):
+                yield asbytes("%d" % i)
+
+        res = np.loadtxt(count())
+        assert_array_equal(res, np.arange(10))
 
 class Testfromregex(TestCase):
     def test_record(self):
@@ -539,10 +689,33 @@ class TestFromTxt(TestCase):
         data[-1] = "99,99"
         kwargs = dict(delimiter=",", names=True, skip_header=5, skip_footer=10)
         test = np.genfromtxt(StringIO(asbytes("\n".join(data))), **kwargs)
-        ctrl = np.array([("%f" % i, "%f" % i, "%f" % i) for i in range(40)],
+        ctrl = np.array([("%f" % i, "%f" % i, "%f" % i) for i in range(41)],
                         dtype=[(_, float) for _ in "ABC"])
         assert_equal(test, ctrl)
 
+    def test_skip_footer_with_invalid(self):
+        import warnings
+        basestr = '1 1\n2 2\n3 3\n4 4\n5  \n6  \n7  \n'
+        warnings.filterwarnings("ignore")
+        # Footer too small to get rid of all invalid values
+        assert_raises(ValueError, np.genfromtxt,
+                      StringIO(basestr), skip_footer=1)
+#        except ValueError:
+#            pass
+        a = np.genfromtxt(StringIO(basestr), skip_footer=1, invalid_raise=False)
+        assert_equal(a, np.array([[1., 1.], [2., 2.], [3., 3.], [4., 4.]]))
+        #
+        a = np.genfromtxt(StringIO(basestr), skip_footer=3)
+        assert_equal(a, np.array([[1., 1.], [2., 2.], [3., 3.], [4., 4.]]))
+        #
+        basestr = '1 1\n2  \n3 3\n4 4\n5  \n6 6\n7 7\n'
+        a = np.genfromtxt(StringIO(basestr), skip_footer=1, invalid_raise=False)
+        assert_equal(a, np.array([[1., 1.], [3., 3.], [4., 4.], [6., 6.]]))
+        a = np.genfromtxt(StringIO(basestr), skip_footer=3, invalid_raise=False)
+        assert_equal(a, np.array([[1., 1.], [3., 3.], [4., 4.]]))
+        warnings.resetwarnings()
+
+
     def test_header(self):
         "Test retrieving a header"
         data = StringIO('gender age weight\nM 64.0 75.0\nF 25.0 60.0')
@@ -686,6 +859,15 @@ M   33  21.99
                       dtype=None)
         assert_raises(ConverterError, np.genfromtxt, s, **kwargs)
 
+    def test_tricky_converter_bug1666(self):
+        "Test some corner case"
+        s = StringIO('q1,2\nq3,4')
+        cnv = lambda s:float(s[1:])
+        test = np.genfromtxt(s, delimiter=',', converters={0:cnv})
+        control = np.array([[1., 2.], [3., 4.]])
+        assert_equal(test, control)
+
+
 
     def test_dtype_with_converters(self):
         dstr = "2009; 23; 46"
@@ -704,10 +886,8 @@ M   33  21.99
         "Test using an explicit dtype with an object"
         from datetime import date
         import time
-        data = asbytes("""
-        1; 2001-01-01
-        2; 2002-01-31
-        """)
+        data = asbytes(""" 1; 2001-01-01
+                           2; 2002-01-31 """)
         ndtype = [('idx', int), ('code', np.object)]
         func = lambda s: strptime(s.strip(), "%Y-%m-%d")
         converters = {1: func}
@@ -746,6 +926,13 @@ M   33  21.99
                             [ 6., 7., 8., 9., 10.]])
         assert_equal(test, control)
 
+    def test_integer_delimiter(self):
+        "Test using an integer for delimiter"
+        data = "  1  2  3\n  4  5 67\n890123  4"
+        test = np.genfromtxt(StringIO(data), delimiter=3)
+        control = np.array([[1, 2, 3], [4, 5, 67], [890, 123, 4]])
+        assert_equal(test, control)
+
 
     def test_missing(self):
         data = StringIO('1,2,3,,5\n')
@@ -820,11 +1007,17 @@ M   33  21.99
                              usecols=('a', 'c'), **kwargs)
         assert_equal(test, ctrl)
 
-
     def test_empty_file(self):
-        "Test that an empty file raises the proper exception"
-        data = StringIO()
-        assert_raises(IOError, np.ndfromtxt, data)
+        "Test that an empty file raises the proper warning."
+        warn_ctx = WarningManager()
+        warn_ctx.__enter__()
+        try:
+            warnings.filterwarnings("ignore", message="genfromtxt: Empty input file:")
+            data = StringIO()
+            test = np.genfromtxt(data)
+            assert_equal(test, np.array([]))
+        finally:
+            warn_ctx.__exit__()
 
 
     def test_fancy_dtype_alt(self):
@@ -845,15 +1038,10 @@ M   33  21.99
                      dtype=dt)
         assert_array_equal(x, a)
 
-
     def test_withmissing(self):
         data = StringIO('A,B\n0,1\n2,N/A')
         kwargs = dict(delimiter=",", missing_values="N/A", names=True)
-        err = np.seterr(invalid='ignore')
-        try:
-            test = np.mafromtxt(data, dtype=None, **kwargs)
-        finally:
-            np.seterr(**err)
+        test = np.mafromtxt(data, dtype=None, **kwargs)
         control = ma.array([(0, 1), (2, -1)],
                            mask=[(False, False), (False, True)],
                            dtype=[('A', np.int), ('B', np.int)])
@@ -861,11 +1049,7 @@ M   33  21.99
         assert_equal(test.mask, control.mask)
         #
         data.seek(0)
-        err = np.seterr(invalid='ignore')
-        try:
-            test = np.mafromtxt(data, **kwargs)
-        finally:
-            np.seterr(**err)
+        test = np.mafromtxt(data, **kwargs)
         control = ma.array([(0, 1), (2, -1)],
                            mask=[(False, False), (False, True)],
                            dtype=[('A', np.float), ('B', np.float)])
@@ -878,12 +1062,8 @@ M   33  21.99
         basekwargs = dict(dtype=None, delimiter=",", names=True,)
         mdtype = [('A', int), ('B', float), ('C', complex)]
         #
-        err = np.seterr(invalid='ignore')
-        try:
-            test = np.mafromtxt(StringIO(data), missing_values="N/A",
-                                **basekwargs)
-        finally:
-            np.seterr(**err)
+        test = np.mafromtxt(StringIO(data), missing_values="N/A",
+                            **basekwargs)
         control = ma.array([(0, 0.0, 0j), (1, -999, 1j),
                             (-9, 2.2, -999j), (3, -99, 3j)],
                             mask=[(0, 0, 0), (0, 1, 0), (0, 0, 1), (0, 0, 0)],
@@ -891,26 +1071,17 @@ M   33  21.99
         assert_equal(test, control)
         #
         basekwargs['dtype'] = mdtype
-        err = np.seterr(invalid='ignore')
-        try:
-            test = np.mafromtxt(StringIO(data),
-                                missing_values={0:-9, 1:-99, 2:-999j},
-                                **basekwargs)
-        finally:
-            np.seterr(**err)
+        test = np.mafromtxt(StringIO(data),
+                            missing_values={0:-9, 1:-99, 2:-999j}, **basekwargs)
         control = ma.array([(0, 0.0, 0j), (1, -999, 1j),
                             (-9, 2.2, -999j), (3, -99, 3j)],
                             mask=[(0, 0, 0), (0, 1, 0), (1, 0, 1), (0, 1, 0)],
                             dtype=mdtype)
         assert_equal(test, control)
         #
-        err = np.seterr(invalid='ignore')
-        try:
-            test = np.mafromtxt(StringIO(data),
-                                missing_values={0:-9, 'B':-99, 'C':-999j},
-                                **basekwargs)
-        finally:
-            np.seterr(**err)
+        test = np.mafromtxt(StringIO(data),
+                            missing_values={0:-9, 'B':-99, 'C':-999j},
+                            **basekwargs)
         control = ma.array([(0, 0.0, 0j), (1, -999, 1j),
                             (-9, 2.2, -999j), (3, -99, 3j)],
                             mask=[(0, 0, 0), (0, 1, 0), (1, 0, 1), (0, 1, 0)],
@@ -1024,7 +1195,7 @@ M   33  21.99
         converters = {4: lambda x:"(%s)" % x}
         kwargs = dict(delimiter=",", converters=converters,
                       dtype=[(_, int) for _ in 'abcde'],)
-        assert_raises(TypeError, np.genfromtxt, mdata, **kwargs)
+        assert_raises(ValueError, np.genfromtxt, mdata, **kwargs)
 
 
     def test_default_field_format(self):
@@ -1131,6 +1302,27 @@ M   33  21.99
                         dtype=[('a', int), ('f0', float), ('f1', int)])
         assert_equal(test, ctrl)
 
+    def test_names_with_usecols_bug1636(self):
+        "Make sure we pick up the right names w/ usecols"
+        data = "A,B,C,D,E\n0,1,2,3,4\n0,1,2,3,4\n0,1,2,3,4"
+        ctrl_names = ("A", "C", "E")
+        test = np.genfromtxt(StringIO(data),
+                             dtype=(int, int, int), delimiter=",",
+                             usecols=(0, 2, 4), names=True)
+        assert_equal(test.dtype.names, ctrl_names)
+        #
+        test = np.genfromtxt(StringIO(data),
+                             dtype=(int, int, int), delimiter=",",
+                             usecols=("A", "C", "E"), names=True)
+        assert_equal(test.dtype.names, ctrl_names)
+        #
+        test = np.genfromtxt(StringIO(data),
+                             dtype=int, delimiter=",",
+                             usecols=("A", "C", "E"), names=True)
+        assert_equal(test.dtype.names, ctrl_names)
+
+
+
     def test_fixed_width_names(self):
         "Test fix-width w/ names"
         data = "    A    B   C\n    0    1 2.3\n   45   67   9."
@@ -1201,6 +1393,32 @@ M   33  21.99
         self.assertTrue(isinstance(test, np.recarray))
         assert_equal(test, control)
 
+    def test_gft_filename(self):
+        # Test that we can load data from a filename as well as a file object
+        data = '0 1 2\n3 4 5'
+        exp_res = np.arange(6).reshape((2,3))
+        assert_array_equal(np.genfromtxt(StringIO(data)), exp_res)
+        f, name = mkstemp()
+        # Thanks to another windows brokeness, we can't use
+        # NamedTemporaryFile: a file created from this function cannot be
+        # reopened by another open call. So we first put the string
+        # of the test reference array, write it to a securely opened file,
+        # which is then read from by the loadtxt function
+        try:
+            os.write(f, asbytes(data))
+            assert_array_equal(np.genfromtxt(name), exp_res)
+        finally:
+            os.close(f)
+            os.unlink(name)
+
+    def test_gft_generator_source(self):
+        def count():
+            for i in range(10):
+                yield asbytes("%d" % i)
+
+        res = np.genfromtxt(count())
+        assert_array_equal(res, np.arange(10))
+
 
 def test_gzip_load():
     a = np.random.random((5, 5))
@@ -1273,5 +1491,20 @@ def test_npzfile_dict():
 
     assert 'x' in list(z.iterkeys())
 
+def test_load_refcount():
+    # Check that objects returned by np.load are directly freed based on
+    # their refcount, rather than needing the gc to collect them.
+
+    f = StringIO()
+    np.savez(f, [1, 2, 3])
+    f.seek(0)
+
+    gc.collect()
+    n_before = len(gc.get_objects())
+    np.load(f)
+    n_after = len(gc.get_objects())
+
+    assert_equal(n_before, n_after)
+
 if __name__ == "__main__":
     run_module_suite()
