@@ -222,6 +222,13 @@ class TestHistogram2d(TestCase):
         H, xed, yed = histogram2d(r, r, (4, 5), range=([0,1], [0,1]))
         assert_array_equal(H, 0)
 
+    def test_empty(self):
+        a, edge1, edge2 = histogram2d([],[], bins=([0,1],[0,1]))
+        assert_array_max_ulp(a, array([[ 0.]]))
+
+        a, edge1, edge2 = histogram2d([], [], bins=4)
+        assert_array_max_ulp(a, np.zeros((4, 4)))
+
 
 class TestTri(TestCase):
     def test_dtype(self):
@@ -232,58 +239,60 @@ class TestTri(TestCase):
         assert_array_equal(tri(3,dtype=bool),out.astype(bool))
 
 
-class TestMaskIndices(TestCase):
-    def test_mask_indices(self):
-        # simple test without offset
-        iu = mask_indices(3, np.triu)
-        a = np.arange(9).reshape(3, 3)
-        yield (assert_array_equal, a[iu], array([0, 1, 2, 4, 5, 8]))
-        # Now with an offset
-        iu1 = mask_indices(3, np.triu, 1)
-        yield (assert_array_equal, a[iu1], array([1, 2, 5]))
+def test_tril_triu():
+    for dtype in np.typecodes['AllFloat'] + np.typecodes['AllInteger']:
+        a = np.ones((2, 2), dtype=dtype)
+        b = np.tril(a)
+        c = np.triu(a)
+        assert_array_equal(b, [[1, 0], [1, 1]])
+        assert_array_equal(c, b.T)
+        # should return the same dtype as the original array
+        assert_equal(b.dtype, a.dtype)
+        assert_equal(c.dtype, a.dtype)
 
 
-class TestTrilIndices(TestCase):
-    def test_tril_indices(self):
-        # indices without and with offset
-        il1 = tril_indices(4)
-        il2 = tril_indices(4, 2)
-
-        a = np.array([[1, 2, 3, 4],
-                      [5, 6, 7, 8],
-                      [9, 10, 11, 12],
-                      [13, 14, 15, 16]])
-
-        # indexing:
-        yield (assert_array_equal, a[il1],
-               array([ 1,  5,  6,  9, 10, 11, 13, 14, 15, 16]) )
-
-        # And for assigning values:
-        a[il1] = -1
-        yield (assert_array_equal, a,
-        array([[-1,  2,  3,  4],
-               [-1, -1,  7,  8],
-               [-1, -1, -1, 12],
-               [-1, -1, -1, -1]]) )
-
-        # These cover almost the whole array (two diagonals right of the main one):
-        a[il2] = -10
-        yield (assert_array_equal, a,
-        array([[-10, -10, -10,   4],
-               [-10, -10, -10, -10],
-               [-10, -10, -10, -10],
-               [-10, -10, -10, -10]]) )
+def test_mask_indices():
+    # simple test without offset
+    iu = mask_indices(3, np.triu)
+    a = np.arange(9).reshape(3, 3)
+    yield (assert_array_equal, a[iu], array([0, 1, 2, 4, 5, 8]))
+    # Now with an offset
+    iu1 = mask_indices(3, np.triu, 1)
+    yield (assert_array_equal, a[iu1], array([1, 2, 5]))
 
 
-class TestTrilIndicesFrom(TestCase):
+def test_tril_indices():
+    # indices without and with offset
+    il1 = tril_indices(4)
+    il2 = tril_indices(4, 2)
 
-    def test_exceptions(self):
-        yield assert_raises(ValueError, tril_indices_from, np.ones((2,)))
-        yield assert_raises(ValueError, tril_indices_from, np.ones((2,2,2)))
-        yield assert_raises(ValueError, tril_indices_from, np.ones((2,3)))
+    a = np.array([[1, 2, 3, 4],
+                  [5, 6, 7, 8],
+                  [9, 10, 11, 12],
+                  [13, 14, 15, 16]])
+
+    # indexing:
+    yield (assert_array_equal, a[il1],
+           array([ 1,  5,  6,  9, 10, 11, 13, 14, 15, 16]) )
+
+    # And for assigning values:
+    a[il1] = -1
+    yield (assert_array_equal, a,
+    array([[-1,  2,  3,  4],
+           [-1, -1,  7,  8],
+           [-1, -1, -1, 12],
+           [-1, -1, -1, -1]]) )
+
+    # These cover almost the whole array (two diagonals right of the main one):
+    a[il2] = -10
+    yield (assert_array_equal, a,
+    array([[-10, -10, -10,   4],
+           [-10, -10, -10, -10],
+           [-10, -10, -10, -10],
+           [-10, -10, -10, -10]]) )
 
 
-class TestTriuIndices(TestCase):
+class TestTriuIndices:
     def test_triu_indices(self):
         iu1 = triu_indices(4)
         iu2 = triu_indices(4, 2)
@@ -300,26 +309,32 @@ class TestTriuIndices(TestCase):
         # And for assigning values:
         a[iu1] = -1
         yield (assert_array_equal, a,
-        array([[-1, -1, -1, -1],
-               [ 5, -1, -1, -1],
-               [ 9, 10, -1, -1],
-               [13, 14, 15, -1]])  )
+               array([[-1, -1, -1, -1],
+                      [ 5, -1, -1, -1],
+                      [ 9, 10, -1, -1],
+                      [13, 14, 15, -1]])  )
 
         # These cover almost the whole array (two diagonals right of the main one):
         a[iu2] = -10
         yield ( assert_array_equal, a,
-        array([[ -1,  -1, -10, -10],
-               [  5,  -1,  -1, -10],
-               [  9,  10,  -1,  -1],
-               [ 13,  14,  15,  -1]]) )
+                array([[ -1,  -1, -10, -10],
+                       [  5,  -1,  -1, -10],
+                       [  9,  10,  -1,  -1],
+                       [ 13,  14,  15,  -1]]) )
 
 
-class TestTriuIndicesFrom(TestCase):
-
+class TestTrilIndicesFrom:
     def test_exceptions(self):
-        yield assert_raises(ValueError, triu_indices_from, np.ones((2,)))
-        yield assert_raises(ValueError, triu_indices_from, np.ones((2,2,2)))
-        yield assert_raises(ValueError, triu_indices_from, np.ones((2,3)))
+        assert_raises(ValueError, tril_indices_from, np.ones((2,)))
+        assert_raises(ValueError, tril_indices_from, np.ones((2,2,2)))
+        assert_raises(ValueError, tril_indices_from, np.ones((2,3)))
+
+
+class TestTriuIndicesFrom:
+    def test_exceptions(self):
+        assert_raises(ValueError, triu_indices_from, np.ones((2,)))
+        assert_raises(ValueError, triu_indices_from, np.ones((2,2,2)))
+        assert_raises(ValueError, triu_indices_from, np.ones((2,3)))
 
 
 if __name__ == "__main__":
