@@ -260,7 +260,7 @@ def iscomplexobj(x):
     False
     >>> np.iscomplexobj(1+0j)
     True
-    np.iscomplexobj([3, 1+0j, True])
+    >>> np.iscomplexobj([3, 1+0j, True])
     True
 
     """
@@ -345,6 +345,7 @@ def nan_to_num(x):
 
     Examples
     --------
+    >>> np.set_printoptions(precision=8)
     >>> x = np.array([np.inf, -np.inf, np.nan, -128, 128])
     >>> np.nan_to_num(x)
     array([  1.79769313e+308,  -1.79769313e+308,   0.00000000e+000,
@@ -610,7 +611,7 @@ def datetime_data(dtype):
 
     if dtype.kind not in ['m','M']:
         raise ValueError, "Not a date-time dtype"
-    
+
     obj = dtype.metadata[METADATA_DTSTR]
     class DATETIMEMETA(ctypes.Structure):
         _fields_ = [('base', ctypes.c_int),
@@ -618,11 +619,17 @@ def datetime_data(dtype):
                     ('den', ctypes.c_int),
                     ('events', ctypes.c_int)]
 
-    func = ctypes.pythonapi.PyCObject_AsVoidPtr
-    func.argtypes = [ctypes.py_object]
-    func.restype = ctypes.c_void_p
-
-    result = func(ctypes.py_object(obj))
+    import sys
+    if sys.version_info[:2] >= (3, 0):
+        func = ctypes.pythonapi.PyCapsule_GetPointer
+        func.argtypes = [ctypes.py_object, ctypes.c_char_p]
+        func.restype = ctypes.c_void_p
+        result = func(ctypes.py_object(obj), ctypes.c_char_p(None))
+    else:
+        func = ctypes.pythonapi.PyCObject_AsVoidPtr
+        func.argtypes = [ctypes.py_object]
+        func.restype = ctypes.c_void_p
+        result = func(ctypes.py_object(obj))
     result = ctypes.cast(ctypes.c_void_p(result), ctypes.POINTER(DATETIMEMETA))
 
     struct = result[0]
@@ -631,8 +638,11 @@ def datetime_data(dtype):
     # FIXME: This needs to be kept consistent with enum in ndarrayobject.h
     from numpy.core.multiarray import DATETIMEUNITS
     obj = ctypes.py_object(DATETIMEUNITS)
-    result = func(obj)
+    if sys.version_info[:2] >= (2,7):
+        result = func(obj, ctypes.c_char_p(None))
+    else:
+        result = func(obj)
     _unitnum2name = ctypes.cast(ctypes.c_void_p(result), ctypes.POINTER(ctypes.c_char_p))
 
     return (_unitnum2name[base], struct.num, struct.den, struct.events)
-    
+
