@@ -205,7 +205,7 @@ def tensorsolve(a, b, axes=None):
 
     See Also
     --------
-    tensordot, tensorinv
+    tensordot, tensorinv, einsum
 
     Examples
     --------
@@ -1297,7 +1297,8 @@ def svd(a, full_matrices=1, compute_uv=1):
     iwork = zeros((8*min(m, n),), fortran_int)
     if isComplexType(t):
         lapack_routine = lapack_lite.zgesdd
-        rwork = zeros((5*min(m, n)*min(m, n) + 5*min(m, n),), real_t)
+        lrwork = min(m,n)*max(5*min(m,n)+7, 2*max(m,n)+2*min(m,n)+1)
+        rwork = zeros((lrwork,), real_t)
         lwork = 1
         work = zeros((lwork,), t)
         results = lapack_routine(option, m, n, a, m, s, u, m, vt, nvt,
@@ -1567,8 +1568,8 @@ def slogdet(a):
 
     Parameters
     ----------
-    a : array_like, shape (M, M)
-        Input array.
+    a : array_like
+        Input array, has to be a square 2-D array.
 
     Returns
     -------
@@ -1580,18 +1581,22 @@ def slogdet(a):
         The natural log of the absolute value of the determinant.
 
     If the determinant is zero, then `sign` will be 0 and `logdet` will be
-    -Inf. In all cases, the determinant is equal to `sign * np.exp(logdet)`.
+    -Inf. In all cases, the determinant is equal to ``sign * np.exp(logdet)``.
+
+    See Also
+    --------
+    det
 
     Notes
     -----
     The determinant is computed via LU factorization using the LAPACK
     routine z/dgetrf.
 
-    .. versionadded:: 2.0.0.
+    .. versionadded:: 1.6.0.
 
     Examples
     --------
-    The determinant of a 2-D array [[a, b], [c, d]] is ad - bc:
+    The determinant of a 2-D array ``[[a, b], [c, d]]`` is ``ad - bc``:
 
     >>> a = np.array([[1, 2], [3, 4]])
     >>> (sign, logdet) = np.linalg.slogdet(a)
@@ -1607,10 +1612,6 @@ def slogdet(a):
     >>> np.linalg.slogdet(np.eye(500) * 0.1)
     (1, -1151.2925464970228)
 
-    See Also
-    --------
-    det
-
     """
     a = asarray(a)
     _assertRank2(a)
