@@ -274,10 +274,17 @@ def lagfromroots(roots) :
         return np.ones(1)
     else :
         [roots] = pu.as_series([roots], trim=False)
-        prd = np.array([1], dtype=roots.dtype)
-        for r in roots:
-            prd = lagsub(lagmulx(prd), r*prd)
-        return prd
+        roots.sort()
+        p = [lagline(-r, 1) for r in roots]
+        n = len(p)
+        while n > 1:
+            m, r = divmod(n, 2)
+            tmp = [lagmul(p[i], p[i+m]) for i in range(m)]
+            if r:
+                tmp[0] = lagmul(tmp[0], p[-1])
+            p = tmp
+            n = m
+        return p[0]
 
 
 def lagadd(c1, c2):
@@ -916,9 +923,16 @@ def lagfit(x, y, deg, rcond=None, full=False, w=None):
     """
     Least squares fit of Laguerre series to data.
 
-    Fit a Laguerre series ``p(x) = p[0] * P_{0}(x) + ... + p[deg] *
-    P_{deg}(x)`` of degree `deg` to points `(x, y)`. Returns a vector of
-    coefficients `p` that minimises the squared error.
+    Return the coefficients of a Laguerre series of degree `deg` that is the
+    least squares fit to the data values `y` given at points `x`. If `y` is
+    1-D the returned coefficients will also be 1-D. If `y` is 2-D multiple
+    fits are done, one for each column of `y`, and the resulting
+    coefficients are stored in the corresponding columns of a 2-D return.
+    The fitted polynomial(s) are in the form
+
+    .. math::  p(x) = c_0 + c_1 * L_1(x) + ... + c_n * L_n(x),
+
+    where `n` is `deg`.
 
     Parameters
     ----------
@@ -969,41 +983,42 @@ def lagfit(x, y, deg, rcond=None, full=False, w=None):
 
     See Also
     --------
+    chebfit, legfit, polyfit, hermfit, hermefit
     lagval : Evaluates a Laguerre series.
-    lagvander : Vandermonde matrix of Laguerre series.
-    polyfit : least squares fit using polynomials.
-    chebfit : least squares fit using Chebyshev series.
+    lagvander : pseudo Vandermonde matrix of Laguerre series.
+    lagweight : Laguerre weight function.
     linalg.lstsq : Computes a least-squares fit from the matrix.
     scipy.interpolate.UnivariateSpline : Computes spline fits.
 
     Notes
     -----
-    The solution are the coefficients ``c[i]`` of the Laguerre series
-    ``P(x)`` that minimizes the squared error
+    The solution is the coefficients of the Laguerre series `p` that
+    minimizes the sum of the weighted squared errors
 
-    ``E = \\sum_j |y_j - P(x_j)|^2``.
+    .. math:: E = \\sum_j w_j^2 * |y_j - p(x_j)|^2,
 
-    This problem is solved by setting up as the overdetermined matrix
-    equation
+    where the :math:`w_j` are the weights. This problem is solved by
+    setting up as the (typically) overdetermined matrix equation
 
-    ``V(x)*c = y``,
+    .. math:: V(x) * c = w * y,
 
-    where ``V`` is the Vandermonde matrix of `x`, the elements of ``c`` are
-    the coefficients to be solved for, and the elements of `y` are the
+    where `V` is the weighted pseudo Vandermonde matrix of `x`, `c` are the
+    coefficients to be solved for, `w` are the weights, and `y` are the
     observed values.  This equation is then solved using the singular value
-    decomposition of ``V``.
+    decomposition of `V`.
 
-    If some of the singular values of ``V`` are so small that they are
+    If some of the singular values of `V` are so small that they are
     neglected, then a `RankWarning` will be issued. This means that the
     coeficient values may be poorly determined. Using a lower order fit
     will usually get rid of the warning.  The `rcond` parameter can also be
     set to a value smaller than its default, but the resulting fit may be
     spurious and have large contributions from roundoff error.
 
-    Fits using Laguerre series are usually better conditioned than fits
-    using power series, but much can depend on the distribution of the
-    sample points and the smoothness of the data. If the quality of the fit
-    is inadequate splines may be a good alternative.
+    Fits using Laguerre series are probably most useful when the data can
+    be approximated by ``sqrt(w(x)) * p(x)``, where `w(x)` is the Laguerre
+    weight. In that case the wieght ``sqrt(w(x[i])`` should be used
+    together with data values ``y[i]/sqrt(w(x[i])``. The weight function is
+    available as `lagweight`.
 
     References
     ----------
@@ -1073,6 +1088,45 @@ def lagfit(x, y, deg, rcond=None, full=False, w=None):
         return c
 
 
+def lagcompanion(cs):
+    """Return the companion matrix of cs.
+
+    The unscaled companion matrix of the Laguerre polynomials is already
+    symmetric when `cs` represents a single Laguerre polynomial, so no
+    further scaling is needed.
+
+    Parameters
+    ----------
+    cs : array_like
+        1-d array of Laguerre series coefficients ordered from low to high
+        degree.
+
+    Returns
+    -------
+    mat : ndarray
+        Companion matrix of dimensions (deg, deg).
+
+    """
+    accprod = np.multiply.accumulate
+    # cs is a trimmed copy
+    [cs] = pu.as_series([cs])
+    if len(cs) < 2:
+        raise ValueError('Series must have maximum degree of at least 1.')
+    if len(cs) == 2:
+        return np.array(1 + cs[0]/cs[1])
+
+    n = len(cs) - 1
+    mat = np.zeros((n, n), dtype=cs.dtype)
+    top = mat.reshape(-1)[1::n+1]
+    mid = mat.reshape(-1)[0::n+1]
+    bot = mat.reshape(-1)[n::n+1]
+    top[...] = -np.arange(1,n)
+    mid[...] = 2.*np.arange(n) + 1.
+    bot[...] = top
+    mat[:,-1] += (cs[:-1]/cs[-1])*n
+    return mat
+
+
 def lagroots(cs):
     """
     Compute the roots of a Laguerre series.
@@ -1122,21 +1176,10 @@ def lagroots(cs):
     if len(cs) == 2 :
         return np.array([1 + cs[0]/cs[1]])
 
-    n = len(cs) - 1
-    cs /= cs[-1]
-    cmat = np.zeros((n,n), dtype=cs.dtype)
-    cmat[0, 0] = 1
-    cmat[1, 0] = -1
-    for i in range(1, n):
-        cmat[i - 1, i] = -i
-        cmat[i, i] = 2*i + 1
-        if i != n - 1:
-            cmat[i + 1, i] = -(i + 1)
-        else:
-            cmat[:, i] += cs[:-1]*(i + 1)
-    roots = la.eigvals(cmat)
-    roots.sort()
-    return roots
+    m = lagcompanion(cs)
+    r = la.eigvals(m)
+    r.sort()
+    return r
 
 
 #
